@@ -24,14 +24,6 @@
 
 #include "kcategorizedview.h"
 
-#include <config-nepomuk.h>
-#ifdef HAVE_NEPOMUK
-#include <nepomuk/global.h>
-#include <nepomuk/resource.h>
-#include <nepomuk/tag.h>
-#include <Soprano/Vocabulary/Xesam>
-#endif
-
 #include <kdatetime.h>
 #include <kdirmodel.h>
 #include <kfileitem.h>
@@ -45,86 +37,124 @@
 #include <QList>
 #include <QSortFilterProxyModel>
 #include <QPainter>
+#include <QPersistentModelIndex>
 #include <QDir>
 #include <QFileInfo>
 
-const char* DolphinModel::m_others = I18N_NOOP2("@title:group Name", "Others");
+const char* const DolphinModel::m_others = I18N_NOOP2("@title:group Name", "Others");
 
-DolphinModel::DolphinModel(QObject* parent)
-    : KDirModel(parent)
+DolphinModel::DolphinModel(QObject* parent) :
+    KDirModel(parent),
+    m_hasVersionData(false),
+    m_revisionHash()
 {
+    setJobTransfersVisible(true);
 }
 
 DolphinModel::~DolphinModel()
 {
 }
 
+bool DolphinModel::setData(const QModelIndex& index, const QVariant& value, int role)
+{
+    if ((index.column() == DolphinModel::Version) && (role == Qt::DecorationRole)) {
+        // TODO: remove data again when items are deleted...
+
+        const QPersistentModelIndex key = index;
+        const KVersionControlPlugin::VersionState state = static_cast<KVersionControlPlugin::VersionState>(value.toInt());
+        if (m_revisionHash.value(key, KVersionControlPlugin::UnversionedVersion) != state) {
+            if (!m_hasVersionData) {
+                connect(this, SIGNAL(rowsRemoved (const QModelIndex&, int, int)),
+                        this, SLOT(slotRowsRemoved(const QModelIndex&, int, int)));
+                m_hasVersionData = true;
+            }
+
+            m_revisionHash.insert(key, state);
+            emit dataChanged(index, index);
+            return true;
+        }
+    }
+
+    return KDirModel::setData(index, value, role);
+}
+
 QVariant DolphinModel::data(const QModelIndex& index, int role) const
 {
     switch (role) {
     case KCategorizedSortFilterProxyModel::CategoryDisplayRole:
         return displayRoleData(index);
+
     case KCategorizedSortFilterProxyModel::CategorySortRole:
         return sortRoleData(index);
+
+    case Qt::DecorationRole:
+        if (index.column() == DolphinModel::Version) {
+            return m_revisionHash.value(index, KVersionControlPlugin::UnversionedVersion);
+        }
+        break;
+
+    case Qt::DisplayRole:
+        switch (index.column()) {
+        case DolphinModel::LinkDest: {
+            const KDirModel *dirModel = qobject_cast<const KDirModel*>(index.model());
+            const KFileItem item = dirModel->itemForIndex(index);
+            return item.linkDest();
+        }
+
+        case DolphinModel::LocalPathOrUrl:
+            const KDirModel *dirModel = qobject_cast<const KDirModel*>(index.model());
+            const KFileItem item = dirModel->itemForIndex(index);
+            return item.localPath();
+            break;
+        }
+        break;
+
     default:
-        return KDirModel::data(index, role);
+        break;
     }
+
+    return KDirModel::data(index, role);
 }
 
-int DolphinModel::columnCount(const QModelIndex &parent) const
+QVariant DolphinModel::headerData(int section, Qt::Orientation orientation, int role) const
+{
+    if ((orientation == Qt::Horizontal) && (role == Qt::DisplayRole)) {
+        switch (section) {
+        case DolphinModel::LinkDest:
+            return i18nc("@title::column", "Link Destination");
+        case DolphinModel::LocalPathOrUrl:
+            return i18nc("@title::column", "Path");
+        default:
+            return KDirModel::headerData(section, orientation, role);
+        }
+    }
+    return QVariant();
+}
+
+int DolphinModel::columnCount(const QModelIndex& parent) const
 {
     return KDirModel::columnCount(parent) + (ExtraColumnCount - ColumnCount);
 }
 
-quint32 DolphinModel::ratingForIndex(const QModelIndex& index)
+void DolphinModel::clearVersionData()
 {
-#ifdef HAVE_NEPOMUK
-    quint32 rating = 0;
-
-    const DolphinModel* dolphinModel = static_cast<const DolphinModel*>(index.model());
-    KFileItem item = dolphinModel->itemForIndex(index);
-    if (!item.isNull()) {
-        const Nepomuk::Resource resource(item.url().url(), Soprano::Vocabulary::Xesam::File());
-        rating = resource.rating();
-    }
-    return rating;
-#else
-    Q_UNUSED(index);
-    return 0;
-#endif
+    m_revisionHash.clear();
+    m_hasVersionData = false;
 }
 
-QString DolphinModel::tagsForIndex(const QModelIndex& index)
+bool DolphinModel::hasVersionData() const
 {
-#ifdef HAVE_NEPOMUK
-    QString tagsString;
+    return m_hasVersionData;
+}
 
-    const DolphinModel* dolphinModel = static_cast<const DolphinModel*>(index.model());
-    KFileItem item = dolphinModel->itemForIndex(index);
-    if (!item.isNull()) {
-        const Nepomuk::Resource resource(item.url().url(), Soprano::Vocabulary::Xesam::File());
-        const QList<Nepomuk::Tag> tags = resource.tags();
-        QStringList stringList;
-        foreach (const Nepomuk::Tag& tag, tags) {
-            stringList.append(tag.label());
-        }
-        stringList.sort();
-
-        foreach (const QString& str, stringList) {
-            tagsString += str;
-            tagsString += ", ";
-        }
-
-        if (!tagsString.isEmpty()) {
-            tagsString.resize(tagsString.size() - 2);
+void DolphinModel::slotRowsRemoved(const QModelIndex& parent, int start, int end)
+{
+    if (m_hasVersionData) {
+        const int column = parent.column();
+        for (int row = start; row <= end; ++row) {
+            m_revisionHash.remove(parent.child(row, column));
         }
     }
-
-    return tagsString;
-#else
-    Q_UNUSED(index);
-    return QString();
-#endif
 }
 
 QVariant DolphinModel::displayRoleData(const QModelIndex& index) const
@@ -218,6 +248,11 @@ QVariant DolphinModel::displayRoleData(const QModelIndex& index) const
         }
 
         if (currentDate.year() == modifiedDate.year() && currentDate.month() == modifiedDate.month()) {
+            if (modifiedWeek > currentWeek) {
+                // use case: modified date = 2010-01-01, current date = 2010-01-22
+                //           modified week = 53,         current week = 3
+                modifiedWeek = 0;
+            }
             switch (currentWeek - modifiedWeek) {
             case 0:
                 switch (daysDistance) {
@@ -324,23 +359,11 @@ QVariant DolphinModel::displayRoleData(const QModelIndex& index) const
         retString = item.mimeComment();
         break;
 
-#ifdef HAVE_NEPOMUK
-    case DolphinModel::Rating: {
-        const quint32 rating = ratingForIndex(index);
-        retString = QString::number(rating);
+    case DolphinModel::Version:
+        retString = "test";
         break;
     }
 
-    case DolphinModel::Tags: {
-        retString = tagsForIndex(index);
-        if (retString.isEmpty()) {
-            retString = i18nc("@title:group Tags", "Not yet tagged");
-        }
-        break;
-    }
-#endif
-    }
-
     return retString;
 }
 
@@ -414,18 +437,6 @@ QVariant DolphinModel::sortRoleData(const QModelIndex& index) const
         }
         break;
 
-#ifdef HAVE_NEPOMUK
-    case DolphinModel::Rating: {
-        retVariant = ratingForIndex(index);
-        break;
-    }
-
-    case DolphinModel::Tags: {
-        retVariant = tagsForIndex(index).count();
-        break;
-    }
-#endif
-
     default:
         break;
     }
