@@ -49,9 +49,10 @@
 #include <ktoggleaction.h>
 #include <kurl.h>
 
+#include "additionalinfoaccessor.h"
 #include "dolphinmodel.h"
 #include "dolphincolumnviewcontainer.h"
-#include "dolphincontroller.h"
+#include "dolphinviewcontroller.h"
 #include "dolphindetailsview.h"
 #include "dolphinfileitemdelegate.h"
 #include "dolphinnewmenuobserver.h"
@@ -62,6 +63,7 @@
 #include "draganddrophelper.h"
 #include "renamedialog.h"
 #include "settings/dolphinsettings.h"
+#include "viewmodecontroller.h"
 #include "viewproperties.h"
 #include "zoomlevelinfo.h"
 #include "dolphindetailsviewexpander.h"
@@ -81,7 +83,6 @@ DolphinView::DolphinView(QWidget* parent,
     QWidget(parent),
     m_active(true),
     m_showPreview(false),
-    m_loadingDirectory(false),
     m_storedCategorizedSorting(false),
     m_tabsForFiles(false),
     m_isContextMenuOpen(false),
@@ -89,7 +90,8 @@ DolphinView::DolphinView(QWidget* parent,
     m_assureVisibleCurrentIndex(false),
     m_mode(DolphinView::IconsView),
     m_topLayout(0),
-    m_controller(0),
+    m_dolphinViewController(0),
+    m_viewModeController(0),
     m_viewAccessor(proxyModel),
     m_selectionModel(0),
     m_selectionChangedTimer(0),
@@ -104,34 +106,38 @@ DolphinView::DolphinView(QWidget* parent,
     m_topLayout->setSpacing(0);
     m_topLayout->setMargin(0);
 
-    m_controller = new DolphinController(this);
-    m_controller->setUrl(url);
+    m_dolphinViewController = new DolphinViewController(this);
 
-    connect(m_controller, SIGNAL(urlChanged(const KUrl&)),
+    m_viewModeController = new ViewModeController(this);
+    m_viewModeController->setUrl(url);
+
+    connect(m_viewModeController, SIGNAL(urlChanged(const KUrl&)),
             this, SIGNAL(urlChanged(const KUrl&)));
 
-    connect(m_controller, SIGNAL(requestContextMenu(const QPoint&, const QList<QAction*>&)),
+    connect(m_dolphinViewController, SIGNAL(requestContextMenu(const QPoint&, const QList<QAction*>&)),
             this, SLOT(openContextMenu(const QPoint&, const QList<QAction*>&)));
-    connect(m_controller, SIGNAL(urlsDropped(const KFileItem&, const KUrl&, QDropEvent*)),
+    connect(m_dolphinViewController, SIGNAL(urlsDropped(const KFileItem&, const KUrl&, QDropEvent*)),
             this, SLOT(dropUrls(const KFileItem&, const KUrl&, QDropEvent*)));
-    connect(m_controller, SIGNAL(sortingChanged(DolphinView::Sorting)),
+    connect(m_dolphinViewController, SIGNAL(sortingChanged(DolphinView::Sorting)),
             this, SLOT(updateSorting(DolphinView::Sorting)));
-    connect(m_controller, SIGNAL(sortOrderChanged(Qt::SortOrder)),
+    connect(m_dolphinViewController, SIGNAL(sortOrderChanged(Qt::SortOrder)),
             this, SLOT(updateSortOrder(Qt::SortOrder)));
-    connect(m_controller, SIGNAL(sortFoldersFirstChanged(bool)),
+    connect(m_dolphinViewController, SIGNAL(sortFoldersFirstChanged(bool)),
             this, SLOT(updateSortFoldersFirst(bool)));
-    connect(m_controller, SIGNAL(additionalInfoChanged(const KFileItemDelegate::InformationList&)),
+    connect(m_dolphinViewController, SIGNAL(additionalInfoChanged(const KFileItemDelegate::InformationList&)),
             this, SLOT(updateAdditionalInfo(const KFileItemDelegate::InformationList&)));
-    connect(m_controller, SIGNAL(itemTriggered(const KFileItem&)),
+    connect(m_dolphinViewController, SIGNAL(itemTriggered(const KFileItem&)),
             this, SLOT(triggerItem(const KFileItem&)));
-    connect(m_controller, SIGNAL(tabRequested(const KUrl&)),
+    connect(m_dolphinViewController, SIGNAL(tabRequested(const KUrl&)),
             this, SIGNAL(tabRequested(const KUrl&)));
-    connect(m_controller, SIGNAL(activated()),
+    connect(m_dolphinViewController, SIGNAL(activated()),
             this, SLOT(activate()));
-    connect(m_controller, SIGNAL(itemEntered(const KFileItem&)),
+    connect(m_dolphinViewController, SIGNAL(itemEntered(const KFileItem&)),
             this, SLOT(showHoverInformation(const KFileItem&)));
-    connect(m_controller, SIGNAL(viewportEntered()),
+    connect(m_dolphinViewController, SIGNAL(viewportEntered()),
             this, SLOT(clearHoverInformation()));
+    connect(m_dolphinViewController, SIGNAL(urlChangeRequested(KUrl)),
+            m_viewModeController, SLOT(setUrl(KUrl)));
 
     KDirLister* dirLister = m_viewAccessor.dirLister();
     connect(dirLister, SIGNAL(redirection(KUrl,KUrl)),
@@ -161,9 +167,9 @@ DolphinView::~DolphinView()
 {
 }
 
-const KUrl& DolphinView::url() const
+KUrl DolphinView::url() const
 {
-    return m_controller->url();
+    return m_viewModeController->url();
 }
 
 KUrl DolphinView::rootUrl() const
@@ -203,7 +209,7 @@ void DolphinView::setActive(bool active)
         emit activated();
     }
 
-    m_controller->indicateActivationChange(active);
+    m_viewModeController->indicateActivationChange(active);
 }
 
 bool DolphinView::isActive() const
@@ -217,7 +223,7 @@ void DolphinView::setMode(Mode mode)
         return; // the wished mode is already set
     }
 
-    const int oldZoomLevel = m_controller->zoomLevel();
+    const int oldZoomLevel = m_viewModeController->zoomLevel();
     m_mode = mode;
 
     // remember the currently selected items, so that they will
@@ -345,35 +351,6 @@ QItemSelectionModel* DolphinView::selectionModel() const
     return m_viewAccessor.itemView()->selectionModel();
 }
 
-void DolphinView::setContentsPosition(int x, int y)
-{
-    QAbstractItemView* view = m_viewAccessor.itemView();
-    Q_ASSERT(view != 0);
-    view->horizontalScrollBar()->setValue(x);
-    view->verticalScrollBar()->setValue(y);
-
-    m_loadingDirectory = false;
-}
-
-void DolphinView::setRestoredContentsPosition(const QPoint& pos)
-{
-    // TODO: This function is called by DolphinViewContainer.
-    // If it makes use of DolphinView::restoreState(...) to restore the
-    // view state in KDE 4.5, this function can be removed.
-    m_restoredContentsPosition = pos;
-}
-
-QPoint DolphinView::contentsPosition() const
-{
-    // TODO: If DolphinViewContainer uses DolphinView::saveState(...) to save the
-    // view state in KDE 4.5, this code can be moved to DolphinView::saveState.
-    QAbstractItemView* view = m_viewAccessor.itemView();
-    Q_ASSERT(view != 0);
-    const int x = view->horizontalScrollBar()->value();
-    const int y = view->verticalScrollBar()->value();
-    return QPoint(x, y);
-}
-
 void DolphinView::setZoomLevel(int level)
 {
     if (level < ZoomLevelInfo::minimumLevel()) {
@@ -383,14 +360,14 @@ void DolphinView::setZoomLevel(int level)
     }
 
     if (level != zoomLevel()) {
-        m_controller->setZoomLevel(level);
+        m_viewModeController->setZoomLevel(level);
         emit zoomLevelChanged(level);
     }
 }
 
 int DolphinView::zoomLevel() const
 {
-    return m_controller->zoomLevel();
+    return m_viewModeController->zoomLevel();
 }
 
 void DolphinView::setSorting(Sorting sorting)
@@ -467,7 +444,7 @@ void DolphinView::refresh()
     m_ignoreViewProperties = false;
 
     const bool oldActivationState = m_active;
-    const int oldZoomLevel = m_controller->zoomLevel();
+    const int oldZoomLevel = m_viewModeController->zoomLevel();
     m_active = true;
 
     createView();
@@ -478,37 +455,9 @@ void DolphinView::refresh()
     updateZoomLevel(oldZoomLevel);
 }
 
-void DolphinView::updateView(const KUrl& url, const KUrl& rootUrl)
-{
-    Q_UNUSED(rootUrl); // TODO: remove after columnview-cleanup has been finished
-
-    if (m_controller->url() != url) {
-        m_controller->setUrl(url); // emits urlChanged, which we forward
-        m_viewAccessor.prepareUrlChange(url);
-        applyViewProperties();
-        loadDirectory(url);
-
-        // When changing the URL there is no need to keep the version
-        // data of the previous URL.
-        m_viewAccessor.dirModel()->clearVersionData();
-
-        emit startedPathLoading(url);
-    }
-
-    // the selection model might have changed in the case of a column view
-    QItemSelectionModel* selectionModel = m_viewAccessor.itemView()->selectionModel();
-    if (m_selectionModel != selectionModel) {
-        disconnect(m_selectionModel, SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
-                   this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
-        m_selectionModel = selectionModel;
-        connect(m_selectionModel, SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
-                this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
-    }
-}
-
 void DolphinView::setNameFilter(const QString& nameFilter)
 {
-    m_controller->setNameFilter(nameFilter);
+    m_viewModeController->setNameFilter(nameFilter);
 }
 
 void DolphinView::calculateItemCount(int& fileCount,
@@ -586,13 +535,35 @@ QString DolphinView::statusBarText() const
 
 QList<QAction*> DolphinView::versionControlActions(const KFileItemList& items) const
 {
-    return m_controller->versionControlActions(items);
+    return m_dolphinViewController->versionControlActions(items);
 }
 
 void DolphinView::setUrl(const KUrl& url)
 {
-    m_newFileNames.clear();
-    updateView(url, KUrl());
+    if (m_viewModeController->url() != url) {
+        m_newFileNames.clear();
+
+        m_viewModeController->setUrl(url); // emits urlChanged, which we forward
+        m_viewAccessor.prepareUrlChange(url);
+        applyViewProperties();
+        loadDirectory(url);
+
+        // When changing the URL there is no need to keep the version
+        // data of the previous URL.
+        m_viewAccessor.dirModel()->clearVersionData();
+
+        emit startedPathLoading(url);
+    }
+
+    // the selection model might have changed in the case of a column view
+    QItemSelectionModel* selectionModel = m_viewAccessor.itemView()->selectionModel();
+    if (m_selectionModel != selectionModel) {
+        disconnect(m_selectionModel, SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
+                   this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
+        m_selectionModel = selectionModel;
+        connect(m_selectionModel, SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
+                this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
+    }
 }
 
 void DolphinView::selectAll()
@@ -762,7 +733,7 @@ void DolphinView::setShowPreview(bool show)
     props.setShowPreview(show);
 
     m_showPreview = show;
-    const int oldZoomLevel = m_controller->zoomLevel();
+    const int oldZoomLevel = m_viewModeController->zoomLevel();
     emit showPreviewChanged();
 
     // Enabling or disabling the preview might change the icon size of the view.
@@ -850,7 +821,7 @@ bool DolphinView::eventFilter(QObject* watched, QEvent* event)
     switch (event->type()) {
     case QEvent::FocusIn:
         if (watched == m_viewAccessor.itemView()) {
-            m_controller->requestActivation();
+            m_dolphinViewController->requestActivation();
         }
         break;
 
@@ -1001,53 +972,20 @@ void DolphinView::updateAdditionalInfo(const KFileItemDelegate::InformationList&
 
 void DolphinView::updateAdditionalInfoActions(KActionCollection* collection)
 {
+    const AdditionalInfoAccessor& infoAccessor = AdditionalInfoAccessor::instance();
+
+    const KFileItemDelegate::InformationList checkedInfos = m_viewAccessor.itemDelegate()->showInformation();
+    const KFileItemDelegate::InformationList infos = infoAccessor.keys();
+
     const bool enable = (m_mode == DolphinView::DetailsView) ||
                         (m_mode == DolphinView::IconsView);
 
-    QAction* showSizeInfo = collection->action("show_size_info");
-    QAction* showDateInfo = collection->action("show_date_info");
-    QAction* showPermissionsInfo = collection->action("show_permissions_info");
-    QAction* showOwnerInfo = collection->action("show_owner_info");
-    QAction* showGroupInfo = collection->action("show_group_info");
-    QAction* showMimeInfo = collection->action("show_mime_info");
-
-    showSizeInfo->setChecked(false);
-    showDateInfo->setChecked(false);
-    showPermissionsInfo->setChecked(false);
-    showOwnerInfo->setChecked(false);
-    showGroupInfo->setChecked(false);
-    showMimeInfo->setChecked(false);
-
-    showSizeInfo->setEnabled(enable);
-    showDateInfo->setEnabled(enable);
-    showPermissionsInfo->setEnabled(enable);
-    showOwnerInfo->setEnabled(enable);
-    showGroupInfo->setEnabled(enable);
-    showMimeInfo->setEnabled(enable);
-
-    foreach (KFileItemDelegate::Information info, m_viewAccessor.itemDelegate()->showInformation()) {
-        switch (info) {
-        case KFileItemDelegate::Size:
-            showSizeInfo->setChecked(true);
-            break;
-        case KFileItemDelegate::ModificationTime:
-            showDateInfo->setChecked(true);
-            break;
-        case KFileItemDelegate::Permissions:
-            showPermissionsInfo->setChecked(true);
-            break;
-        case KFileItemDelegate::Owner:
-            showOwnerInfo->setChecked(true);
-            break;
-        case KFileItemDelegate::OwnerAndGroup:
-            showGroupInfo->setChecked(true);
-            break;
-        case KFileItemDelegate::FriendlyMimeType:
-            showMimeInfo->setChecked(true);
-            break;
-        default:
-            break;
-        }
+    foreach (const KFileItemDelegate::Information& info, infos) {
+        const QString name = infoAccessor.actionCollectionName(info, AdditionalInfoAccessor::AdditionalInfoType);
+        QAction* action = collection->action(name);
+        Q_ASSERT(action != 0);
+        action->setEnabled(enable);
+        action->setChecked(checkedInfos.contains(info));
     }
 }
 
@@ -1066,21 +1004,14 @@ bool DolphinView::isTabsForFilesEnabled() const
     return m_tabsForFiles;
 }
 
-void DolphinView::activateItem(const KUrl& url)
-{
-    // TODO: If DolphinViewContainer uses DolphinView::restoreState(...) to restore the
-    // view state in KDE 4.5, this function can be removed.
-    m_activeItemUrl = url;
-}
-
 bool DolphinView::itemsExpandable() const
 {
     return m_viewAccessor.itemsExpandable();
 }
 
-void DolphinView::restoreState(QDataStream &stream)
+void DolphinView::restoreState(QDataStream& stream)
 {
-     // current item
+    // current item
     stream >> m_activeItemUrl;
 
     // view position
@@ -1090,8 +1021,7 @@ void DolphinView::restoreState(QDataStream &stream)
     QSet<KUrl> urlsToExpand;
     stream >> urlsToExpand;
     const DolphinDetailsViewExpander* expander = m_viewAccessor.setExpandedUrls(urlsToExpand);
-
-    if (expander) {
+    if (expander != 0) {
         m_expanderActive = true;
         connect (expander, SIGNAL(completed()), this, SLOT(slotLoadingCompleted()));
     }
@@ -1100,26 +1030,29 @@ void DolphinView::restoreState(QDataStream &stream)
     }
 }
 
-void DolphinView::saveState(QDataStream &stream)
+void DolphinView::saveState(QDataStream& stream)
 {
     // current item
     KFileItem currentItem;
     const QAbstractItemView* view = m_viewAccessor.itemView();
 
-    if(view) {
+    if (view != 0) {
         const QModelIndex proxyIndex = view->currentIndex();
         const QModelIndex dirModelIndex = m_viewAccessor.proxyModel()->mapToSource(proxyIndex);
         currentItem = m_viewAccessor.dirModel()->itemForIndex(dirModelIndex);
     }
 
     KUrl currentUrl;
-    if (!currentItem.isNull())
+    if (!currentItem.isNull()) {
         currentUrl = currentItem.url();
+    }
 
     stream << currentUrl;
 
     // view position
-    stream << contentsPosition();
+    const int x = view->horizontalScrollBar()->value();
+    const int y = view->verticalScrollBar()->value();
+    stream << QPoint(x, y);
 
     // expanded folders (only relevant for the details view - the set will be empty in other view modes)
     stream << m_viewAccessor.expandedUrls();
@@ -1145,21 +1078,6 @@ void DolphinView::selectAndScrollToCreatedItem()
     m_createdItemUrl = KUrl();
 }
 
-void DolphinView::emitContentsMoved()
-{
-    // TODO: If DolphinViewContainer uses DolphinView::saveState(...) to save the
-    // view state in KDE 4.5, the contentsMoved signal might not be needed anymore,
-    // depending on how the implementation is done.
-    // In that case, the code in contentsPosition() can be moved to saveState().
-
-    // only emit the contents moved signal if no directory loading is ongoing
-    // (this would reset the contents position always to (0, 0))
-    if (!m_loadingDirectory) {
-        const QPoint pos(contentsPosition());
-        emit contentsMoved(pos.x(), pos.y());
-    }
-}
-
 void DolphinView::showHoverInformation(const KFileItem& item)
 {
     emit requestItemInfo(item);
@@ -1208,7 +1126,6 @@ void DolphinView::slotDirListerCompleted()
 void DolphinView::slotLoadingCompleted()
 {
     m_expanderActive = false;
-    m_loadingDirectory = false;
 
     if (!m_activeItemUrl.isEmpty()) {
         // assure that the current item remains visible
@@ -1267,9 +1184,6 @@ void DolphinView::loadDirectory(const KUrl& url, bool reload)
         return;
     }
 
-    m_loadingDirectory = true;
-    m_expanderActive = false;
-
     KDirLister* dirLister = m_viewAccessor.dirLister();
     dirLister->openUrl(url, reload ? KDirLister::Reload : KDirLister::NoFlags);
 }
@@ -1284,7 +1198,7 @@ void DolphinView::applyViewProperties()
 
     const Mode mode = props.viewMode();
     if (m_mode != mode) {
-        const int oldZoomLevel = m_controller->zoomLevel();
+        const int oldZoomLevel = m_viewModeController->zoomLevel();
 
         m_mode = mode;
         createView();
@@ -1338,7 +1252,7 @@ void DolphinView::applyViewProperties()
     const bool showPreview = props.showPreview();
     if (showPreview != m_showPreview) {
         m_showPreview = showPreview;
-        const int oldZoomLevel = m_controller->zoomLevel();
+        const int oldZoomLevel = m_viewModeController->zoomLevel();
         emit showPreviewChanged();
 
         // Enabling or disabling the preview might change the icon size of the view.
@@ -1360,14 +1274,14 @@ void DolphinView::createView()
     deleteView();
 
     Q_ASSERT(m_viewAccessor.itemView() == 0);
-    m_viewAccessor.createView(this, m_controller, m_mode);
+    m_viewAccessor.createView(this, m_dolphinViewController, m_viewModeController, m_mode);
 
     QAbstractItemView* view = m_viewAccessor.itemView();
     Q_ASSERT(view != 0);
     view->installEventFilter(this);
     view->viewport()->installEventFilter(this);
 
-    m_controller->setItemView(view);
+    m_dolphinViewController->setItemView(view);
 
     // When changing the view mode, the selection is lost due to reinstantiating
     // a new item view with a custom selection model. Pass the ownership of the
@@ -1381,11 +1295,6 @@ void DolphinView::createView()
     connect(m_selectionModel, SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
             this, SLOT(slotSelectionChanged(QItemSelection, QItemSelection)));
 
-    connect(view->verticalScrollBar(), SIGNAL(valueChanged(int)),
-            this, SLOT(emitContentsMoved()));
-    connect(view->horizontalScrollBar(), SIGNAL(valueChanged(int)),
-            this, SLOT(emitContentsMoved()));
-
     setFocusProxy(m_viewAccessor.layoutTarget());
     m_topLayout->insertWidget(1, m_viewAccessor.layoutTarget());
 }
@@ -1406,12 +1315,8 @@ void DolphinView::deleteView()
 
         // disconnect all signal/slots
         disconnect(view);
-        m_controller->disconnect(view);
+        m_viewModeController->disconnect(view);
         view->disconnect();
-        disconnect(view->verticalScrollBar(), SIGNAL(valueChanged(int)),
-                   this, SLOT(emitContentsMoved()));
-        disconnect(view->horizontalScrollBar(), SIGNAL(valueChanged(int)),
-                   this, SLOT(emitContentsMoved()));
 
         m_viewAccessor.deleteView();
     }
@@ -1427,7 +1332,7 @@ void DolphinView::updateZoomLevel(int oldZoomLevel)
 {
     const int newZoomLevel = ZoomLevelInfo::zoomLevelForIconSize(m_viewAccessor.itemView()->iconSize());
     if (oldZoomLevel != newZoomLevel) {
-        m_controller->setZoomLevel(newZoomLevel);
+        m_viewModeController->setZoomLevel(newZoomLevel);
         emit zoomLevelChanged(newZoomLevel);
     }
 }
@@ -1473,22 +1378,31 @@ DolphinView::ViewAccessor::~ViewAccessor()
 }
 
 void DolphinView::ViewAccessor::createView(QWidget* parent,
-                                           DolphinController* controller,
+                                           DolphinViewController* dolphinViewController,
+                                           const ViewModeController* viewModeController,
                                            Mode mode)
 {
     Q_ASSERT(itemView() == 0);
 
     switch (mode) {
     case IconsView:
-        m_iconsView = new DolphinIconsView(parent, controller, m_proxyModel);
+        m_iconsView = new DolphinIconsView(parent,
+                                           dolphinViewController,
+                                           viewModeController,
+                                           m_proxyModel);
         break;
 
     case DetailsView:
-        m_detailsView = new DolphinDetailsView(parent, controller, m_proxyModel);
+        m_detailsView = new DolphinDetailsView(parent,
+                                               dolphinViewController,
+                                               viewModeController,
+                                               m_proxyModel);
         break;
 
     case ColumnView:
-        m_columnsContainer = new DolphinColumnViewContainer(parent, controller);
+        m_columnsContainer = new DolphinColumnViewContainer(parent,
+                                                            dolphinViewController,
+                                                            viewModeController);
         break;
 
     default:
@@ -1532,11 +1446,6 @@ void DolphinView::ViewAccessor::prepareUrlChange(const KUrl& url)
     if (m_columnsContainer != 0) {
         m_columnsContainer->showColumn(url);
     }
-
-    if (!m_detailsViewExpander.isNull()) {
-        // stop expanding items in the current folder
-        m_detailsViewExpander->stop();
-    }
 }
 
 QAbstractItemView* DolphinView::ViewAccessor::itemView() const
@@ -1597,6 +1506,11 @@ QSet<KUrl> DolphinView::ViewAccessor::expandedUrls() const
 const DolphinDetailsViewExpander* DolphinView::ViewAccessor::setExpandedUrls(const QSet<KUrl>& urlsToExpand)
 {
     if ((m_detailsView != 0) && m_detailsView->itemsExpandable() && !urlsToExpand.isEmpty()) {
+        // Check if another expander is already active and stop it if necessary.
+        if(!m_detailsViewExpander.isNull()) {
+            m_detailsViewExpander->stop();
+        }
+
         m_detailsViewExpander = new DolphinDetailsViewExpander(m_detailsView, urlsToExpand);
         return m_detailsViewExpander;
     }
@@ -1632,15 +1546,23 @@ KDirLister* DolphinView::ViewAccessor::dirLister() const
 
 void DolphinView::slotRedirection(const KUrl& oldUrl, const KUrl& newUrl)
 {
-    emit redirection(oldUrl, newUrl);
-    m_controller->redirectToUrl(newUrl); // #186947
+    if (oldUrl.equals(url(), KUrl::CompareWithoutTrailingSlash)) {
+        emit redirection(oldUrl, newUrl);
+        m_viewModeController->redirectToUrl(newUrl); // #186947
+    }
 }
 
 void DolphinView::restoreContentsPosition()
 {
     if (!m_restoredContentsPosition.isNull()) {
-        setContentsPosition(m_restoredContentsPosition.x(), m_restoredContentsPosition.y());
+        const int x = m_restoredContentsPosition.x();
+        const int y = m_restoredContentsPosition.y();
         m_restoredContentsPosition = QPoint();
+
+        QAbstractItemView* view = m_viewAccessor.itemView();
+        Q_ASSERT(view != 0);
+        view->horizontalScrollBar()->setValue(x);
+        view->verticalScrollBar()->setValue(y);
     }
 }
 
