@@ -22,21 +22,16 @@
 #include "dolphinmodel.h"
 #include "dolphinsortfilterproxymodel.h"
 
+#include "filemetadatatooltip.h"
 #include <kicon.h>
 #include <kio/previewjob.h>
-#include <kseparator.h>
-
-#include "panels/information/kmetadatawidget.h"
-#include "tooltips/ktooltip.h"
+#include <kwindowsystem.h>
 
 #include <QApplication>
 #include <QDesktopWidget>
-#include <QHBoxLayout>
-#include <QLabel>
 #include <QScrollArea>
 #include <QScrollBar>
 #include <QTimer>
-#include <QVBoxLayout>
 
 ToolTipManager::ToolTipManager(QAbstractItemView* parent,
                                DolphinSortFilterProxyModel* model) :
@@ -44,36 +39,53 @@ ToolTipManager::ToolTipManager(QAbstractItemView* parent,
     m_view(parent),
     m_dolphinModel(0),
     m_proxyModel(model),
-    m_timer(0),
-    m_previewTimer(0),
+    m_prepareToolTipTimer(0),
+    m_startPreviewJobTimer(0),
     m_waitOnPreviewTimer(0),
+    m_showToolTipDelayedTimer(0),
+    m_fileMetaDataToolTip(0),
     m_item(),
     m_itemRect(),
     m_generatingPreview(false),
     m_hasDefaultIcon(false),
     m_previewPixmap()
 {
+    static FileMetaDataToolTip* sharedToolTip = 0;
+    if (sharedToolTip == 0) {
+        sharedToolTip = new FileMetaDataToolTip();
+        // TODO: Using K_GLOBAL_STATIC would be preferable to maintain the
+        // instance, but the cleanup of KFileMetaDataWidget at this stage does
+        // not work.
+    }
+    m_fileMetaDataToolTip = sharedToolTip;
+    
     m_dolphinModel = static_cast<DolphinModel*>(m_proxyModel->sourceModel());
     connect(parent, SIGNAL(entered(const QModelIndex&)),
             this, SLOT(requestToolTip(const QModelIndex&)));
     connect(parent, SIGNAL(viewportEntered()),
             this, SLOT(hideToolTip()));
 
-    m_timer = new QTimer(this);
-    m_timer->setSingleShot(true);
-    connect(m_timer, SIGNAL(timeout()),
-            this, SLOT(prepareToolTip()));
-
-    m_previewTimer = new QTimer(this);
-    m_previewTimer->setSingleShot(true);
-    connect(m_previewTimer, SIGNAL(timeout()),
-            this, SLOT(startPreviewJob()));
-
+    // Initialize timers
+    m_prepareToolTipTimer = new QTimer(this);
+    m_prepareToolTipTimer->setSingleShot(true);
+    m_prepareToolTipTimer->setInterval(500);
+    connect(m_prepareToolTipTimer, SIGNAL(timeout()), this, SLOT(prepareToolTip()));   
+    
+    m_startPreviewJobTimer = new QTimer(this);
+    m_startPreviewJobTimer->setSingleShot(true);
+    m_startPreviewJobTimer->setInterval(200);
+    connect(m_startPreviewJobTimer, SIGNAL(timeout()), this, SLOT(startPreviewJob()));
+    
     m_waitOnPreviewTimer = new QTimer(this);
     m_waitOnPreviewTimer->setSingleShot(true);
-    connect(m_waitOnPreviewTimer, SIGNAL(timeout()),
-            this, SLOT(prepareToolTip()));
+    m_waitOnPreviewTimer->setInterval(250);     
+    connect(m_waitOnPreviewTimer, SIGNAL(timeout()), this, SLOT(prepareToolTip()));
 
+    m_showToolTipDelayedTimer = new QTimer(this);
+    m_showToolTipDelayedTimer->setSingleShot(true);
+    m_showToolTipDelayedTimer->setInterval(100);     
+    connect(m_showToolTipDelayedTimer, SIGNAL(timeout()), this, SLOT(showToolTip()));
+    
     // When the mousewheel is used, the items don't get a hovered indication
     // (Qt-issue #200665). To assure that the tooltip still gets hidden,
     // the scrollbars are observed.
@@ -115,12 +127,11 @@ bool ToolTipManager::eventFilter(QObject* watched, QEvent* event)
 
 void ToolTipManager::requestToolTip(const QModelIndex& index)
 {
-    // only request a tooltip for the name column and when no selection or
+    hideToolTip();
+    
+    // Only request a tooltip for the name column and when no selection or
     // drag & drop operation is done (indicated by the left mouse button)
     if ((index.column() == DolphinModel::Name) && !(QApplication::mouseButtons() & Qt::LeftButton)) {
-        m_waitOnPreviewTimer->stop();
-        KToolTip::hideTip();
-
         m_itemRect = m_view->visualRect(index);
         const QPoint pos = m_view->viewport()->mapToGlobal(m_itemRect.topLeft());
         m_itemRect.moveTo(pos);
@@ -128,37 +139,37 @@ void ToolTipManager::requestToolTip(const QModelIndex& index)
         const QModelIndex dirIndex = m_proxyModel->mapToSource(index);
         m_item = m_dolphinModel->itemForIndex(dirIndex);
 
-        // only start the previewJob when the mouse has been over this item for 200 milliseconds,
-        // this prevents a lot of useless preview jobs when passing rapidly over a lot of items
-        m_previewTimer->start(200);
+        // Only start the previewJob when the mouse has been over this item for 200 milliseconds.
+        // This prevents a lot of useless preview jobs when passing rapidly over a lot of items.
+        m_startPreviewJobTimer->start();
         m_previewPixmap = QPixmap();
         m_hasDefaultIcon = false;
 
-        m_timer->start(500);
-    } else {
-        hideToolTip();
+        m_prepareToolTipTimer->start();
     }
 }
 
 void ToolTipManager::hideToolTip()
 {
-    m_timer->stop();
-    m_previewTimer->stop();
+    m_prepareToolTipTimer->stop();
+    m_startPreviewJobTimer->stop();
     m_waitOnPreviewTimer->stop();
-    KToolTip::hideTip();
+    m_showToolTipDelayedTimer->stop();
+
+    m_fileMetaDataToolTip->hide();
 }
 
 void ToolTipManager::prepareToolTip()
 {
     if (m_generatingPreview) {
-        m_waitOnPreviewTimer->start(250);
+        m_waitOnPreviewTimer->start();
     }
 
     if (!m_previewPixmap.isNull()) {
-        showToolTip(m_previewPixmap);
+        showToolTipDelayed(m_previewPixmap);
     } else if (!m_hasDefaultIcon) {
         const QPixmap image(KIcon(m_item.iconName()).pixmap(128, 128));
-        showToolTip(image);
+        showToolTipDelayed(image);
         m_hasDefaultIcon = true;
     }
 }
@@ -179,7 +190,7 @@ void ToolTipManager::setPreviewPix(const KFileItem& item,
                                    const QPixmap& pixmap)
 {
     if ((m_item.url() != item.url()) || pixmap.isNull()) {
-        // an old preview or an invalid preview has been received
+        // An old preview or an invalid preview has been received
         previewFailed();
     } else {
         m_previewPixmap = pixmap;
@@ -192,19 +203,11 @@ void ToolTipManager::previewFailed()
     m_generatingPreview = false;
 }
 
-
-void ToolTipManager::showToolTip(const QPixmap& pixmap)
+void ToolTipManager::showToolTip()
 {
-    if (QApplication::mouseButtons() & Qt::LeftButton) {
-        return;
-    }
-
-    QWidget* tip = createTipContent(pixmap);
-
-    // calculate the x- and y-position of the tooltip
-    const QSize size = tip->sizeHint();
     const QRect desktop = QApplication::desktop()->screenGeometry(m_itemRect.bottomRight());
-
+    const QSize size = m_fileMetaDataToolTip->sizeHint();
+    
     // m_itemRect defines the area of the item, where the tooltip should be
     // shown. Per default the tooltip is shown in the bottom right corner.
     // If the tooltip content exceeds the desktop borders, it must be assured that:
@@ -215,11 +218,9 @@ void ToolTipManager::showToolTip(const QPixmap& pixmap)
     const bool hasRoomAbove   = (m_itemRect.top()    - size.height() >= desktop.top());
     const bool hasRoomBelow   = (m_itemRect.bottom() + size.height() <= desktop.bottom());
     if (!hasRoomAbove && !hasRoomBelow && !hasRoomToLeft && !hasRoomToRight) {
-        delete tip;
-        tip = 0;
         return;
     }
-
+    
     int x = 0;
     int y = 0;
     if (hasRoomBelow || hasRoomAbove) {
@@ -227,64 +228,50 @@ void ToolTipManager::showToolTip(const QPixmap& pixmap)
         if (x + size.width() >= desktop.right()) {
             x = desktop.right() - size.width();
         }
-        y = hasRoomBelow ? m_itemRect.bottom() : m_itemRect.top() - size.height();
+        if (hasRoomBelow) {
+            y = m_itemRect.bottom() + 1;
+        } else {
+            y = m_itemRect.top() - size.height();
+        }
     } else {
         Q_ASSERT(hasRoomToLeft || hasRoomToRight);
-        x = hasRoomToRight ? m_itemRect.right() : m_itemRect.left() - size.width();
+        if (hasRoomToRight) {
+            x = m_itemRect.right() + 1;
+        } else {
+            x = m_itemRect.left() - size.width();
+        }
 
         // Put the tooltip at the bottom of the screen. The x-coordinate has already
         // been adjusted, so that no overlapping with m_itemRect occurs.
         y = desktop.bottom() - size.height();
     }
-
-    // the ownership of tip is transferred to KToolTip
-    KToolTip::showTip(QPoint(x, y), tip);
+    
+    m_fileMetaDataToolTip->move(qMax(x, 0), qMax(y, 0));
+    m_fileMetaDataToolTip->show();
 }
 
-QWidget* ToolTipManager::createTipContent(const QPixmap& pixmap) const
+void ToolTipManager::showToolTipDelayed(const QPixmap& pixmap)
 {
-    QWidget* tipContent = new QWidget();
+    if (QApplication::mouseButtons() & Qt::LeftButton) {
+        return;
+    }
+    
+    m_fileMetaDataToolTip->setPreview(pixmap);
+    m_fileMetaDataToolTip->setName(m_item.text());
+    m_fileMetaDataToolTip->setItems(KFileItemList() << m_item);
 
-    // add pixmap
-    QLabel* pixmapLabel = new QLabel(tipContent);
-    pixmapLabel->setPixmap(pixmap);
-    pixmapLabel->setFixedSize(pixmap.size());
-
-    // add item name
-    QLabel* nameLabel = new QLabel(tipContent);
-    nameLabel->setText(m_item.text());
-    nameLabel->setWordWrap(true);
-    QFont font = nameLabel->font();
-    font.setBold(true);
-    nameLabel->setFont(font);
-    nameLabel->setAlignment(Qt::AlignHCenter);
-    nameLabel->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-
-    // add meta data
-    KMetaDataWidget* metaDataWidget = new KMetaDataWidget(tipContent);
-    metaDataWidget->setForegroundRole(QPalette::ToolTipText);
-    metaDataWidget->setItem(m_item);
-    metaDataWidget->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-    metaDataWidget->setReadOnly(true);
-
-    // the stretchwidget allows the metadata widget to be top aligned and fills
-    // the remaining vertical space
-    QWidget* stretchWidget = new QWidget(tipContent);
-    stretchWidget->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
-
-    QWidget* textContainer = new QWidget(tipContent);
-    QVBoxLayout* textLayout = new QVBoxLayout(textContainer);
-    textLayout->addWidget(nameLabel);
-    textLayout->addWidget(new KSeparator());
-    textLayout->addWidget(metaDataWidget);
-    textLayout->addWidget(stretchWidget);
-
-    QHBoxLayout* tipLayout = new QHBoxLayout(tipContent);
-    tipLayout->setMargin(0);
-    tipLayout->addWidget(pixmapLabel);
-    tipLayout->addWidget(textContainer);
-
-    return tipContent;
+    // Because one tooltip instance is reused, the size hint always depends on the previous
+    // content (QWidgets don't update their layout geometry if they are invisible). To
+    // assure having a consistent size without relayout flickering, the tooltip is opened
+    // on an invisible position first. This gives the layout system some time to asynchronously
+    // update the content. Sadly this only works with compositing enabled.
+    if (KWindowSystem::compositingActive()) {
+        const QRect desktop = QApplication::desktop()->screenGeometry(m_itemRect.bottomRight());
+        m_fileMetaDataToolTip->move(desktop.bottomRight());
+        m_fileMetaDataToolTip->show();
+    }
+    
+    m_showToolTipDelayedTimer->start(); // Calls ToolTipManager::showToolTip()
 }
 
 #include "tooltipmanager.moc"
