@@ -18,11 +18,12 @@
 */
 
 #include "bookmarklistview.h"
-#include "bookmarkmodel.h"
-#include "toplevel.h"
+#include "globalbookmarkmanager.h"
+#include "kbookmarkmodel/model.h"
+#include "toplevel.h" // for KEBApp
 #include "settings.h"
-#include "commands.h"
-#include "treeitem_p.h"
+#include "kbookmarkmodel/commands.h"
+
 #include <QtGui/QHeaderView>
 #include <QtGui/QItemSelection>
 #include <QtGui/QMenu>
@@ -32,22 +33,8 @@
 
 #include <kdebug.h>
 
-BookmarkView::BookmarkView( QWidget * parent )
-    :QTreeView( parent )
-{
-    setAcceptDrops(true);
-}
-
-BookmarkView::~BookmarkView()
-{
-
-}
-
-
-/* ----------- */
-
 BookmarkFolderView::BookmarkFolderView( BookmarkListView * view, QWidget * parent )
-    :BookmarkView( parent ), mview(view)
+    : KBookmarkView(parent), mview(view)
 {
     mmodel = new BookmarkFolderViewFilterModel(parent);
     mmodel->setSourceModel(view->model());
@@ -55,9 +42,8 @@ BookmarkFolderView::BookmarkFolderView( BookmarkListView * view, QWidget * paren
     header()->setVisible(false);
     setRootIsDecorated(false);
     setDropIndicatorShown(true);
-    expandAll();
     setCurrentIndex( mmodel->index(0,0, QModelIndex()));
-    
+
     connect(mmodel, SIGNAL(modelReset()), this, SLOT(slotReset()));
 }
 
@@ -74,13 +60,13 @@ void BookmarkFolderView::selectionChanged ( const QItemSelection & deselected, c
         mview->setRootIndex( mmodel->mapToSource(list.at(0)) );
     else
         mview->setRootIndex( QModelIndex());
-    BookmarkView::selectionChanged( deselected, selected);
+    KBookmarkView::selectionChanged(deselected, selected);
 }
 
 void BookmarkFolderView::slotReset()
 {
     setCurrentIndex( mmodel->index(0,0, QModelIndex()));
-    expandAll();
+    loadFoldedState();
 }
 
 KBookmark BookmarkFolderView::bookmarkForIndex(const QModelIndex & idx) const
@@ -94,7 +80,7 @@ KBookmark BookmarkFolderView::bookmarkForIndex(const QModelIndex & idx) const
 
 
 BookmarkListView::BookmarkListView( QWidget * parent )
-    :BookmarkView( parent )
+    : KBookmarkView(parent)
 {
     setDragEnabled(true);
 }
@@ -102,7 +88,7 @@ BookmarkListView::BookmarkListView( QWidget * parent )
 
 void BookmarkListView::setModel(QAbstractItemModel * model)
 {
-    BookmarkView::setModel(model);
+    KBookmarkView::setModel(model);
 }
 
 KBookmarkModel* BookmarkListView::bookmarkModel() const
@@ -130,7 +116,7 @@ void BookmarkListView::contextMenuEvent ( QContextMenuEvent * e )
 
     QMenu* popup;
     if( !index.isValid()
-       || (bk.address() == CurrentMgr::self()->root().address())
+       || (bk.address() == GlobalBookmarkManager::self()->root().address())
        || (bk.isGroup())) //FIXME add empty folder padder
     {
         popup = KEBApp::self()->popupMenuFactory("popup_folder");
@@ -143,7 +129,7 @@ void BookmarkListView::contextMenuEvent ( QContextMenuEvent * e )
         popup->popup(e->globalPos());
 }
 
-void BookmarkListView::loadColumnSetting() 
+void BookmarkListView::loadColumnSetting()
 {
     header()->resizeSection(KEBApp::NameColumn, KEBSettings::name());
     header()->resizeSection(KEBApp::UrlColumn, KEBSettings::uRL());
@@ -151,7 +137,7 @@ void BookmarkListView::loadColumnSetting()
     header()->resizeSection(KEBApp::StatusColumn, KEBSettings::status());
 }
 
-void BookmarkListView::saveColumnSetting() 
+void BookmarkListView::saveColumnSetting()
 {
     KEBSettings::setName( header()->sectionSize(KEBApp::NameColumn));
     KEBSettings::setURL( header()->sectionSize(KEBApp::UrlColumn));
@@ -214,10 +200,13 @@ bool BookmarkFolderViewFilterModel::filterAcceptsColumn ( int source_column, con
     return (source_column == 0);
 }
 
+Q_DECLARE_METATYPE(KBookmark)
+
 bool BookmarkFolderViewFilterModel::filterAcceptsRow ( int source_row, const QModelIndex & source_parent ) const
 {
     QModelIndex index = sourceModel()->index(source_row, 0, source_parent);
-    return static_cast<TreeItem *>(index.internalPointer())->bookmark().isGroup();
+    const KBookmark bk = index.data(KBookmarkModel::KBookmarkRole).value<KBookmark>();
+    return bk.isGroup();
 }
 
 #include "bookmarklistview.moc"
