@@ -19,6 +19,8 @@
 
 #include "kfindtreeview.h"
 
+#include "kfinddlg.h"
+
 #include <QtCore/QTextStream>
 #include <QtCore/QTextCodec>
 #include <QtCore/QFileInfo>
@@ -40,12 +42,13 @@
 #include <kio/netaccess.h>
 #include <kio/copyjob.h>
 #include <kio/deletejob.h>
+#include <kjobuidelegate.h>
 
 #include <konq_operations.h>
-#include <knewmenu.h>
+#include <knewfilemenu.h>
 
 // Permission strings
-static const char* perm[4] = {
+static const char* const perm[4] = {
   I18N_NOOP( "Read-write" ),
   I18N_NOOP( "Read-only" ),
   I18N_NOOP( "Write-only" ),
@@ -309,9 +312,10 @@ bool KFindSortFilterProxyModel::lessThan(const QModelIndex &left, const QModelIn
 
 //BEGIN KFindTreeView
 
-KFindTreeView::KFindTreeView( QWidget *parent )
+KFindTreeView::KFindTreeView( QWidget *parent,  KfindDlg * findDialog )
     : QTreeView( parent ) ,
-    m_contextMenu(0)
+    m_contextMenu(0),
+    m_kfindDialog(findDialog)
 {
     //Configure model and proxy model
     m_model = new KFindItemModel( this );
@@ -338,6 +342,7 @@ KFindTreeView::KFindTreeView( QWidget *parent )
                 
     //Generate popup menu actions
     m_actionCollection = new KActionCollection( this );
+    m_actionCollection->addAssociatedWidget(this);
 
     KAction * open = KStandardAction::open(this, SLOT( slotExecuteSelected() ), this);
     m_actionCollection->addAction( "file_open", open );
@@ -494,9 +499,7 @@ void KFindTreeView::saveResults()
         }
 
         file.close();
-        KMessageBox::information(parentWidget(),
-                    i18n("Results were saved to file\n")+
-                    filename);
+        m_kfindDialog->setStatusMsg(i18nc("%1=filename", "Results were saved to: %1", filename));
     }
 }
 
@@ -591,7 +594,7 @@ void KFindTreeView::contextMenuRequested( const QPoint & p)
         delete m_contextMenu;
         m_contextMenu = 0;
     }
-    m_contextMenu = new KonqPopupMenu( fileList, KUrl(), *m_actionCollection, new KNewMenu( m_actionCollection, this, "new_menu"), 0, flags, this, 0, actionGroups);
+    m_contextMenu = new KonqPopupMenu( fileList, KUrl(), *m_actionCollection, new KNewFileMenu( m_actionCollection, "new_menu", this), 0, flags, this, 0, actionGroups);
 
     m_contextMenu->exec( this->mapToGlobal( p ) );
 }
@@ -617,30 +620,30 @@ KUrl::List KFindTreeView::selectedUrls()
 void KFindTreeView::deleteSelectedFiles()
 {
     KUrl::List uris = selectedUrls();
-    
+    if ( uris.isEmpty() ) {
+        return;
+    }
+
     bool done = KonqOperations::askDeleteConfirmation( uris, KonqOperations::DEL, KonqOperations::FORCE_CONFIRMATION, this );
     if ( done )
     {
-        KIO::del( uris );
-        
-        //This should be done by KDirWatch integration in the main dialog, but it could fail?
-        Q_FOREACH( const KUrl & url, uris )
-            m_model->removeItem( url );
+        KJob * deleteJob = KIO::del( uris );
+        deleteJob->uiDelegate()->setAutoErrorHandlingEnabled(true);
     }
 }
 
 void KFindTreeView::moveToTrashSelectedFiles()
 {
     KUrl::List uris = selectedUrls();
-    
+    if ( uris.isEmpty() ) {
+        return;
+    }
+
     bool done = KonqOperations::askDeleteConfirmation( uris, KonqOperations::TRASH, KonqOperations::FORCE_CONFIRMATION, this );
     if ( done )
     {
-        KIO::trash( uris );
-
-        //This should be done by KDirWatch integration in the main dialog, but it could fail?
-        Q_FOREACH( const KUrl & url, uris )
-            m_model->removeItem( url );
+        KJob * trashJob = KIO::trash( uris );
+        trashJob->uiDelegate()->setAutoErrorHandlingEnabled(true);
     }
 }
 
