@@ -64,7 +64,6 @@ KAppearanceOptions::KAppearanceOptions(QWidget *parent, const QVariantList&)
   QGroupBox *box = new QGroupBox( i18n("Images"),mainTab);
   l->addWidget(box);
   QFormLayout *fl= new QFormLayout(box);
-  fl->setSpacing(KDialog::spacingHint());
 
   m_pAutoLoadImagesCheckBox = new QCheckBox( i18n( "A&utomatically load images"), this );
   m_pAutoLoadImagesCheckBox->setWhatsThis( i18n( "<html>If this box is checked, Konqueror will"
@@ -151,7 +150,6 @@ KAppearanceOptions::KAppearanceOptions(QWidget *parent, const QVariantList&)
     fonts.append(empty);
 
   QVBoxLayout *lay = new QVBoxLayout(fontsTab);
-  lay->setSpacing(KDialog::spacingHint());
 
   QGroupBox* gb = new QGroupBox( i18n("Font Si&ze"));
   lay->addWidget(gb);
@@ -163,8 +161,8 @@ KAppearanceOptions::KAppearanceOptions(QWidget *parent, const QVariantList&)
   m_minSize->setRange( 2, 30 );
   connect( m_minSize, SIGNAL( valueChanged( int ) ), this, SLOT( slotMinimumFontSize( int ) ) );
   connect( m_minSize, SIGNAL( valueChanged( int ) ), this, SLOT( changed() ) );
-  m_minSize->setWhatsThis( i18n( "Konqueror will never display text smaller than "
-                                    "this size,<br />overriding any other settings." ) );
+  m_minSize->setWhatsThis( "<qt>" + i18n( "Konqueror will never display text smaller than "
+                                    "this size,<br />overriding any other settings." ) + "</qt>" );
 
   m_MedSize = new KIntNumInput( fSize,m_minSize );
   fl->addRow(i18n( "&Medium font size:"  ),m_MedSize);
@@ -225,7 +223,7 @@ KAppearanceOptions::KAppearanceOptions(QWidget *parent, const QVariantList&)
 
 
 
-  m_pFontSizeAdjust = new QSpinBox( this );
+  m_pFontSizeAdjust = new KIntSpinBox( this );
   m_pFontSizeAdjust->setRange( -5, 5 );
   m_pFontSizeAdjust->setSingleStep( 1 );
   fl->addRow(i18n( "Font &size adjustment for this encoding:" ), m_pFontSizeAdjust);
