@@ -164,12 +164,17 @@ void KonqCombo::init( KCompletion *completion )
     setAutoDeleteCompletionObject( false );
     setCompletionMode( completion->completionMode() );
 
+    // We use Ctrl+T for new tab, so we need something else for substring completion
+    // TODO: how to make that shortcut configurable? If we add a KAction we need to
+    // call the KLineEdit code, which we can't do. Well, we could send a keyevent...
+    setKeyBinding(KCompletionBase::SubstringCompletion, KShortcut(Qt::Key_F7));
+
     loadItems();
 }
 
 void KonqCombo::setURL( const QString& url )
 {
-    //kDebug(1202) << url << "returnPressed=" << m_returnPressed;
+    //kDebug() << url << "returnPressed=" << m_returnPressed;
     setTemporary( url );
 
     if ( m_returnPressed ) { // Really insert...
@@ -190,7 +195,7 @@ void KonqCombo::setTemporary( const QString& text )
 
 void KonqCombo::setTemporary( const QString& url, const QPixmap& pix )
 {
-    //kDebug(1202) << url << "temporary=" << temporary;
+    //kDebug() << url << "temporary=" << temporary;
 
     // Insert a temporary item when we don't have one yet
     if ( count() == 0 )
@@ -208,7 +213,7 @@ void KonqCombo::setTemporary( const QString& url, const QPixmap& pix )
 
 void KonqCombo::removeDuplicates( int index )
 {
-    //kDebug(1202) << "starting index= " << index;
+    //kDebug() << "starting index= " << index;
 
     QString url(temporaryItem());
     if (url.endsWith('/'))
@@ -229,7 +234,7 @@ void KonqCombo::removeDuplicates( int index )
 // called via DBUS in all instances
 void KonqCombo::insertPermanent( const QString& url )
 {
-    //kDebug(1202) << "url=" << url;
+    //kDebug() << "url=" << url;
     saveState();
     setTemporary( url );
     m_permanent = true;
@@ -249,7 +254,7 @@ void KonqCombo::applyPermanent()
 
         QString item = temporaryItem();
         insertItem( KonqPixmapProvider::self()->pixmapFor( item ), item, 1, titleOfURL( item ) );
-        //kDebug(1202) << url;
+        //kDebug() << url;
 
         // Remove all duplicates starting from index = 2
         removeDuplicates( 2 );
@@ -274,7 +279,7 @@ void KonqCombo::updateItem( const QPixmap& pix, const QString& t, int index, con
         (!itemIcon(index).isNull() && itemIcon(index).pixmap(iconSize()).serialNumber() == pix.serialNumber()))
         return;
 
-    // kDebug(1202) << "item=" << t << "index=" << index;
+    // kDebug() << "item=" << t << "index=" << index;
 
     setItemText( index, t );
     setItemIcon( index, pix );
@@ -482,7 +487,7 @@ void KonqCombo::selectWord(QKeyEvent *e)
             count++;
                   if( allow_space_break && text[pos].isSpace() )
                       break;
-        } while( pos < (int) text.length() && chars.indexOf(text[pos]) == -1 );
+        } while( pos < text.length() && chars.indexOf(text[pos]) == -1 );
 
         if( e->modifiers() & Qt::ShiftModifier ) {
             edit->cursorForward(true, count+1);
@@ -826,6 +831,8 @@ void KonqComboLineEdit::mouseDoubleClickEvent( QMouseEvent *e )
     KLineEdit::mouseDoubleClickEvent( e );
 }
 
+// KDE5 TODO: get rid of this duplicated code, and just reimplement completionBox()
+
 void KonqComboLineEdit::setCompletedItems( const QStringList& items, bool )
 {
     QString txt;
@@ -846,31 +853,22 @@ void KonqComboLineEdit::setCompletedItems( const QStringList& items, bool )
 
         if ( completionbox->isVisible() ) {
 
+            // This code is copied from KLineEdit::setCompletedItems
             QListWidgetItem* currentItem = completionbox->currentItem();
-            bool wasSelected = false;
-            QString currentSelection;
 
+            QString currentSelection;
             if ( currentItem != 0 ) {
-                wasSelected = currentItem->isSelected();
                 currentSelection = currentItem->text();
             }
 
             completionbox->setItems( items );
-            QList<QListWidgetItem*> matchedItems = completionbox->findItems
-                                ( currentSelection, Qt::MatchExactly );
+            const QList<QListWidgetItem*> matchedItems = completionbox->findItems(currentSelection, Qt::MatchExactly);
+            QListWidgetItem* matchedItem = matchedItems.isEmpty() ? 0 : matchedItems.first();
 
-            QListWidgetItem* item = matchedItems.isEmpty() ? 0 : matchedItems.first();
-
-            if( !item || !wasSelected )
-            {
-                wasSelected = false;
-                item = completionbox->item( 0 );
-            }
-            if ( item ) {
-                completionbox->blockSignals( true );
-                completionbox->setCurrentItem( item );
-                item->setSelected(wasSelected);
-                completionbox->blockSignals( false );
+            if (matchedItem) {
+                const bool blocked = completionbox->blockSignals(true);
+                completionbox->setCurrentItem(matchedItem);
+                completionbox->blockSignals(blocked);
             }
         }
         else { // completion box not visible yet -> show it
@@ -920,7 +918,7 @@ void KonqComboCompletionBox::setItems( const QStringList& items )
 
         for ( ; it != itEnd; ++it) {
             if ( rowIndex < count() ) {
-                const bool changed = ((KonqListWidgetItem*)item(rowIndex))->reuse( *it );
+                const bool changed = (static_cast<KonqListWidgetItem*>(item(rowIndex)))->reuse( *it );
                 dirty = dirty || changed;
             }
             else {
