@@ -89,7 +89,7 @@ KonqFrameBase::FrameType frameTypeFromString( const QString& str )
 KonqFrame::KonqFrame( QWidget* parent, KonqFrameContainerBase *parentContainer )
     : QWidget ( parent )
 {
-   //kDebug(1202) << "KonqFrame::KonqFrame()";
+   //kDebug() << "KonqFrame::KonqFrame()";
 
    m_pLayout = 0L;
    m_pView = 0L;
@@ -105,7 +105,7 @@ KonqFrame::KonqFrame( QWidget* parent, KonqFrameContainerBase *parentContainer )
 
 KonqFrame::~KonqFrame()
 {
-   //kDebug(1202) << "KonqFrame::~KonqFrame() " << this;
+    //kDebug() << this;
 }
 
 bool KonqFrame::isActivePart()
@@ -153,7 +153,7 @@ void KonqFrame::copyHistory( KonqFrameBase *other )
 
 void KonqFrame::attachWidget(QWidget* widget)
 {
-   //kDebug(1202) << "KonqFrame::attachInternal()";
+   //kDebug() << "KonqFrame::attachInternal()";
    delete m_pLayout;
 
    m_pLayout = new QVBoxLayout( this );
@@ -190,14 +190,14 @@ void KonqFrame::setView( KonqView* child )
 
 void KonqFrame::setTitle( const QString &title , QWidget* /*sender*/)
 {
-  //kDebug(1202) << "KonqFrame::setTitle( " << title << " )";
+  //kDebug() << "KonqFrame::setTitle( " << title << " )";
   m_title = title;
   if (m_pParentContainer) m_pParentContainer->setTitle( title , this);
 }
 
 void KonqFrame::setTabIcon( const KUrl &url, QWidget* /*sender*/ )
 {
-  //kDebug(1202) << "KonqFrame::setTabIcon( " << url << " )";
+  //kDebug() << "KonqFrame::setTabIcon( " << url << " )";
   if (m_pParentContainer) m_pParentContainer->setTabIcon( url, this );
 }
 
@@ -231,8 +231,13 @@ void KonqFrame::slotRemoveView()
 
 void KonqFrame::activateChild()
 {
-  if (m_pView && !m_pView->isPassiveMode() )
-    m_pView->mainWindow()->viewManager()->setActivePart( part() );
+    if (m_pView && !m_pView->isPassiveMode() ) {
+        m_pView->mainWindow()->viewManager()->setActivePart( part() );
+
+        if (m_pView->url().isEmpty() || m_pView->url() == "about:blank") {
+            m_pView->mainWindow()->focusLocationBar(); // #84867 usability improvement
+        }
+    }
 }
 
 KonqView* KonqFrame::childView() const
