@@ -241,10 +241,6 @@ void KonqFrameTabs::insertChildFrame( KonqFrameBase* frame, int index )
     connect(this, SIGNAL(currentChanged(int)),
             this, SLOT(slotCurrentChanged(int)), Qt::UniqueConnection);
 
-    if (m_rightWidget) {
-      m_rightWidget->setEnabled( m_childFrameList.count() > 1 );
-    }
-
     if (KonqView* activeChildView = frame->activeChildView()) {
       activeChildView->setCaption( activeChildView->caption() );
       activeChildView->setTabIcon( activeChildView->url() );
@@ -260,8 +256,6 @@ void KonqFrameTabs::childFrameRemoved( KonqFrameBase * frame )
   if (frame) {
     removeTab(indexOf(frame->asQWidget()));
     m_childFrameList.removeAll(frame);
-    if (m_rightWidget)
-      m_rightWidget->setEnabled( m_childFrameList.count()>1 );
     if (count() == 1)
       updateTabBarVisibility();
   }
@@ -318,15 +312,11 @@ void KonqFrameTabs::slotContextMenu( QWidget *w, const QPoint &p )
   m_popupActions["reload"]->setEnabled( true );
   m_popupActions["duplicatecurrenttab"]->setEnabled( true );
   m_popupActions["breakoffcurrenttab"]->setEnabled( tabCount > 1 );
-  m_popupActions["removecurrenttab"]->setEnabled( tabCount > 1 );
+  m_popupActions["removecurrenttab"]->setEnabled( true );
   m_popupActions["othertabs"]->setEnabled( true );
   m_popupActions["closeothertabs"]->setEnabled( true );
 
-  // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a
-  // class hierarchy and it could crash one day, but I haven't checked
-  // setWorkingTab so I don't know if it can handle nulls.
-
-  m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
+  m_pViewManager->mainWindow()->setWorkingTab( indexOf(w) );
   m_pPopupMenu->exec( p );
 }
 
@@ -366,11 +356,8 @@ void KonqFrameTabs::refreshSubPopupMenuTab()
 
 void KonqFrameTabs::slotCloseRequest( QWidget *w )
 {
-  if ( m_childFrameList.count() > 1 ) {
-    // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a class hierarchy and it could crash one day, but I haven't checked setWorkingTab so I don't know if it can handle nulls.
-    m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
+    m_pViewManager->mainWindow()->setWorkingTab( indexOf(w) );
     emit removeTabPopup();
-  }
 }
 
 void KonqFrameTabs::slotSubPopupMenuTabActivated( QAction *action )
@@ -390,24 +377,19 @@ void KonqFrameTabs::slotMouseMiddleClick()
   }
 }
 
-void KonqFrameTabs::slotMouseMiddleClick( QWidget *w )
+void KonqFrameTabs::slotMouseMiddleClick(QWidget *w)
 {
-  if ( m_MouseMiddleClickClosesTab ) {
-    if ( m_childFrameList.count() > 1 ) {
-      // Yes, I know this is an unchecked dynamic_cast - I'm casting sideways in a class hierarchy and it could crash one day, but I haven't checked setWorkingTab so I don't know if it can handle nulls.
-      m_pViewManager->mainWindow()->setWorkingTab( dynamic_cast<KonqFrameBase*>(w) );
-      emit removeTabPopup();
+    if (m_MouseMiddleClickClosesTab) {
+        slotCloseRequest(w);
+    } else {
+        KUrl filteredURL(KonqMisc::konqFilteredURL(this, QApplication::clipboard()->text(QClipboard::Selection)));
+        if (!filteredURL.isEmpty()) {
+            KonqFrameBase* frame = dynamic_cast<KonqFrameBase*>(w);
+            if (frame) {
+                m_pViewManager->mainWindow()->openUrl(frame->activeChildView(), filteredURL);
+            }
+        }
     }
-  }
-  else {
-  KUrl filteredURL ( KonqMisc::konqFilteredURL( this, QApplication::clipboard()->text(QClipboard::Selection ) ) );
-  if ( !filteredURL.isEmpty() ) {
-    KonqFrameBase* frame = dynamic_cast<KonqFrameBase*>(w);
-    if (frame) {
-      m_pViewManager->mainWindow()->openUrl( frame->activeChildView(), filteredURL );
-    }
-  }
-  }
 }
 
 void KonqFrameTabs::slotTestCanDecode(const QDragMoveEvent *e, bool &accept /* result */)
@@ -543,7 +525,7 @@ void KonqFrameTabs::slotCurrentChanged( int index )
     const KColorScheme colorScheme(QPalette::Active, KColorScheme::Window);
     setTabTextColor(index, colorScheme.foreground(KColorScheme::NormalText).color());
 
-    KonqFrameBase* currentFrame = dynamic_cast<KonqFrameBase*>(widget( index ));
+    KonqFrameBase* currentFrame = tabAt(index);
     if (currentFrame && !m_pViewManager->isLoadingProfile()) {
         m_pActiveChild = currentFrame;
         currentFrame->activateChild();
@@ -552,19 +534,15 @@ void KonqFrameTabs::slotCurrentChanged( int index )
     m_pViewManager->mainWindow()->linkableViewCountChanged();
 }
 
-/**
- * Returns the index position of the tab that contains (directly or indirectly) the frame @p frame,
- * or -1 if the frame is not in the tab widget.
- */
-KonqFrameBase* KonqFrameTabs::tabContaining(KonqFrameBase* frame) const
+int KonqFrameTabs::tabIndexContaining(KonqFrameBase* frame) const
 {
     KonqFrameBase* frameBase = frame;
     while (frameBase && frameBase->parentContainer() != this)
         frameBase = frameBase->parentContainer();
     if (frameBase)
-        return frameBase;
+        return indexOf(frameBase->asQWidget());
     else
-        return 0;
+        return -1;
 }
 
 int KonqFrameTabs::tabWhereActive(KonqFrameBase* frame) const
@@ -610,4 +588,14 @@ void KonqFrameTabs::replaceChildFrame(KonqFrameBase* oldFrame, KonqFrameBase* ne
     setCurrentIndex(index);
 }
 
+KonqFrameBase* KonqFrameTabs::tabAt(int index) const
+{
+    return dynamic_cast<KonqFrameBase*>(widget(index));
+}
+
+KonqFrameBase* KonqFrameTabs::currentTab() const
+{
+    return tabAt(currentIndex());
+}
+
 #include "konqtabs.moc"
