@@ -19,7 +19,7 @@
 
 // Own
 #include "ProcessInfo.h"
-#include "config-konsole.h"
+#include <config-konsole.h>
 
 // Unix
 #include <sys/socket.h>
@@ -53,7 +53,7 @@
 #endif
 
 #if defined(Q_OS_FREEBSD)
-#include <sys/sysctl.h>
+#include <sys/sysctl.h> //krazy:exclude=includes
 #include <sys/types.h>
 #include <sys/user.h>
 #include <libutil.h>
@@ -732,7 +732,7 @@ private:
         int managementInfoBase[4];
         size_t mibLength;
         struct kinfo_proc* kInfoProc;
-        struct stat statInfo;
+        KDE_struct_stat statInfo;
 
         // Find the tty device of 'pid' (Example: /dev/ttys001)
         managementInfoBase[0] = CTL_KERN;
@@ -789,14 +789,17 @@ private:
 
     virtual bool readArguments(int pid)
     {
+        Q_UNUSED(pid);
         return false;
     }
     virtual bool readCurrentDir(int pid)
     {
+        Q_UNUSED(pid);
         return false;
     }
     virtual bool readEnvironment(int pid)
     {
+        Q_UNUSED(pid);
         return false;
     }
 } ;
