@@ -18,6 +18,7 @@
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
     02110-1301  USA.
 */
+//krazy:excludeall=qclasses
 
 // Own
 #include "ViewContainer.h"
@@ -259,6 +260,7 @@ ViewContainerTabBar::ViewContainerTabBar(QWidget* parent,TabbedViewContainer* co
     , _dropIndicatorIndex(-1)
     , _drawIndicatorDisabled(false)
 {
+    setElideMode(Qt::ElideLeft);
 }
 void ViewContainerTabBar::setDropIndicator(int index, bool drawDisabled)
 {
@@ -394,9 +396,29 @@ void ViewContainerTabBar::dropEvent(QDropEvent* event)
         event->ignore();
 }
 
+/* Try to provide a reasonable tab width:
+ * If there are less than 5 tabs, let the tabs fill out the tabbar.
+ * Otherwise, check the tab text size and the minimumTabWidth value.
+ * The minimumTabWidth is to prevent the tabs from getting too
+ * small such that no scroll indicators will appear.
+*/
 QSize ViewContainerTabBar::tabSizeHint(int index) const
 {
-     return QTabBar::tabSizeHint(index);
+    const int fillTabCount = 5;
+    const int minimumTabWidth = 150; // Arbitrary value
+    int tabBarWidth = rect().width();
+    int tabCount = count();
+    int tabMaxWidth;
+    if (tabCount <= fillTabCount)
+        tabMaxWidth = tabBarWidth / (tabCount + 0.5);
+    else
+        tabMaxWidth = tabBarWidth / (fillTabCount + 0.5);
+
+    int tabTextWidth = fontMetrics().width(tabText(index));
+    // qBound ( minimum, value, max )
+    QSize size (qBound(minimumTabWidth, tabTextWidth, tabMaxWidth),
+                QTabBar::tabSizeHint(index).height());
+    return size;
 }
 QPixmap ViewContainerTabBar::dragDropPixmap(int tab) 
 {
@@ -423,6 +445,7 @@ QPixmap ViewContainerTabBar::dragDropPixmap(int tab)
 }
 TabbedViewContainer::TabbedViewContainer(NavigationPosition position , QObject* parent) 
 : ViewContainer(position,parent)
+   , _contextMenuTabIndex(0)
 {
     _containerWidget = new QWidget;
     _stackWidget = new QStackedWidget();
@@ -447,29 +470,29 @@ TabbedViewContainer::TabbedViewContainer(NavigationPosition position , QObject*
     connect( _tabBar , SIGNAL(tabDoubleClicked(int)) , this , SLOT(tabDoubleClicked(int)) );
     connect( _tabBar , SIGNAL(newTabRequest()) , this , SIGNAL(newViewRequest()) );
     connect( _tabBar , SIGNAL(wheelDelta(int)) , this , SLOT(wheelScrolled(int)) );
-    connect( _tabBar , SIGNAL(closeRequest(int)) , this , SLOT(closeTab(int)) );
+    connect( _tabBar , SIGNAL(tabCloseRequested(int)) , this , SLOT(closeTab(int)) );
     connect( _tabBar , SIGNAL(initiateDrag(int)) , this , SLOT(startTabDrag(int)) );
+    connect( _tabBar, SIGNAL(contextMenu(int, const QPoint&)), this,
+            SLOT(openTabContextMenu(int, const QPoint&)) );
 
     connect( _newTabButton , SIGNAL(clicked()) , this , SIGNAL(newViewRequest()) );
     connect( _closeTabButton , SIGNAL(clicked()) , this , SLOT(closeCurrentTab()) );
 
     _layout = new TabbedViewContainerLayout;
     _layout->setSpacing(0);
-    _layout->setMargin(0);
+    _layout->setContentsMargins(0, 0, 0, 0);
     _tabBarLayout = new QHBoxLayout;
     _tabBarLayout->setSpacing(0);
-    _tabBarLayout->setMargin(0);
+    _tabBarLayout->setContentsMargins(0, 0, 0, 0);
     _tabBarLayout->addWidget(_newTabButton);
     _tabBarLayout->addWidget(_tabBar);
     _tabBarLayout->addWidget(_closeTabButton); 
-    _tabBarSpacer = new QSpacerItem(0,TabBarSpace);
 
     _layout->addWidget(_stackWidget);
     searchBar()->setParent(_containerWidget);
     if ( position == NavigationPositionTop )
     {
         _layout->insertLayout(0,_tabBarLayout);
-        _layout->insertItemAt(0,_tabBarSpacer);
         _layout->insertWidget(-1,searchBar());
         _tabBar->setShape(QTabBar::RoundedNorth);
     }
@@ -477,13 +500,27 @@ TabbedViewContainer::TabbedViewContainer(NavigationPosition position , QObject*
     {
         _layout->insertWidget(-1,searchBar());
         _layout->insertLayout(-1,_tabBarLayout);
-        _layout->insertItemAt(-1,_tabBarSpacer);
         _tabBar->setShape(QTabBar::RoundedSouth);
     }
     else
         Q_ASSERT(false); // position not supported
 
     _containerWidget->setLayout(_layout);
+ 
+    _contextPopupMenu = new KMenu(_tabBar);
+
+    _contextPopupMenu->addAction(KIcon("tab-detach"),
+        i18nc("@action:inmenu", "&Detach Tab"), this,
+        SLOT(tabContextMenuDetachTab()));
+
+    _contextPopupMenu->addAction(KIcon(),
+        i18nc("@action:inmenu", "&Rename Tab..."), this,
+        SLOT(tabContextMenuRenameTab()));
+
+    _contextPopupMenu->addAction(KIcon("tab-close"),
+        i18nc("@action:inmenu", "&Close Tab"), this,
+        SLOT(tabContextMenuCloseTab()));
+
 }
 void TabbedViewContainer::setNewViewMenu(QMenu* menu)
 {
@@ -520,14 +557,6 @@ void TabbedViewContainer::setTabBarVisible(bool visible)
     _tabBar->setVisible(visible);
     _newTabButton->setVisible(visible && (features() & QuickNewView));
     _closeTabButton->setVisible(visible && (features() & QuickCloseView));
-    if ( visible )
-    {
-        _tabBarSpacer->changeSize(0,TabBarSpace);
-    }
-    else
-    {
-        _tabBarSpacer->changeSize(0,0);
-    } 
 }
 QList<ViewContainer::NavigationPosition> TabbedViewContainer::supportedNavigationPositions() const
 {
@@ -535,9 +564,9 @@ QList<ViewContainer::NavigationPosition> TabbedViewContainer::supportedNavigatio
 }
 void TabbedViewContainer::navigationPositionChanged(NavigationPosition position)
 {
-    // this method assumes that there are only three items 
+    // this method assumes that there are only two items
     // in the layout
-    Q_ASSERT( _layout->count() == 4 );
+    Q_ASSERT( _layout->count() == 3 );
 
     // index of stack widget in the layout when tab bar is at the bottom
     const int StackIndexWithTabBottom = 0;
@@ -546,11 +575,9 @@ void TabbedViewContainer::navigationPositionChanged(NavigationPosition position)
             && _layout->indexOf(_stackWidget) == StackIndexWithTabBottom )
     {
         _layout->removeItem(_tabBarLayout);
-        _layout->removeItem(_tabBarSpacer);
         _layout->removeWidget(searchBar());
 
         _layout->insertLayout(0,_tabBarLayout);
-        _layout->insertItemAt(0,_tabBarSpacer);
         _layout->insertWidget(-1,searchBar());
         _tabBar->setShape(QTabBar::RoundedNorth);
     }
@@ -558,12 +585,10 @@ void TabbedViewContainer::navigationPositionChanged(NavigationPosition position)
             && _layout->indexOf(_stackWidget) != StackIndexWithTabBottom )
     {
         _layout->removeItem(_tabBarLayout);
-        _layout->removeItem(_tabBarSpacer);
         _layout->removeWidget(searchBar());
 
         _layout->insertWidget(-1,searchBar());
         _layout->insertLayout(-1,_tabBarLayout);
-        _layout->insertItemAt(-1,_tabBarSpacer);
         _tabBar->setShape(QTabBar::RoundedSouth);
     }
 }
@@ -624,10 +649,39 @@ void TabbedViewContainer::startTabDrag(int tab)
         removeView(view);
     }
 }
-void TabbedViewContainer::tabDoubleClicked(int tab)
+
+void TabbedViewContainer::tabDoubleClicked(int index)
 {
-    viewProperties( views()[tab] )->rename();
+    renameTab(index);
 }
+
+void TabbedViewContainer::renameTab(int index)
+{
+    viewProperties(views()[index])->rename();
+}
+
+void TabbedViewContainer::openTabContextMenu(int index, const QPoint& pos)
+{
+    _contextMenuTabIndex = index;
+
+    _contextPopupMenu->exec(pos);
+}
+
+void TabbedViewContainer::tabContextMenuCloseTab()
+{
+    closeTab(_contextMenuTabIndex);
+}
+
+void TabbedViewContainer::tabContextMenuDetachTab()
+{
+    emit detachTab(this, _stackWidget->widget(_contextMenuTabIndex));
+}
+
+void TabbedViewContainer::tabContextMenuRenameTab()
+{
+    renameTab(_contextMenuTabIndex);
+}
+
 void TabbedViewContainer::moveViewWidget( int fromIndex , int toIndex )
 {
     QString text = _tabBar->tabText(fromIndex);
@@ -742,22 +796,11 @@ void TabbedViewContainer::updateActivity(ViewProperties* item)
 
 void TabbedViewContainer::updateTitle(ViewProperties* item)
 {
-    // prevent tab titles from becoming overly-long as this limits the number
-    // of tabs which can fit in the tab bar.  
-    //
-    // if the view's title is overly long then trim it and select the 
-    // right-most 20 characters (assuming they contain the most useful
-    // information) and insert an elide at the front
-    const int MAX_TAB_TEXT_LENGTH = 20;
-
     QListIterator<QWidget*> iter(widgetsForItem(item));
     while ( iter.hasNext() )
     {
         const int index = _stackWidget->indexOf( iter.next() );
-
         QString tabText = item->title();
-        if (tabText.count() > MAX_TAB_TEXT_LENGTH)
-            tabText = tabText.right(MAX_TAB_TEXT_LENGTH).prepend("...");
 
         _tabBar->setTabText( index , tabText );
     }
@@ -783,7 +826,7 @@ StackedViewContainer::StackedViewContainer(QObject* parent)
     searchBar()->setParent(_containerWidget);
     layout->addWidget(searchBar());
     layout->addWidget(_stackWidget);
-    layout->setMargin(0);
+    layout->setContentsMargins(0, 0, 0, 0);
 }
 StackedViewContainer::~StackedViewContainer()
 {
@@ -813,145 +856,9 @@ void StackedViewContainer::removeViewWidget( QWidget* view )
     const int index = _stackWidget->indexOf(view);
 
     Q_ASSERT( index != -1);
+    Q_UNUSED(index);
 
     _stackWidget->removeWidget(view);
 }
 
-ListViewContainer::ListViewContainer(NavigationPosition position,QObject* parent)
-    : ViewContainer(position,parent)
-{
-    _splitter = new QSplitter;
-    _listWidget = new ProfileListWidget(_splitter);
-
-    QWidget *contentArea = new QWidget(_splitter);
-    QVBoxLayout *layout = new QVBoxLayout(contentArea);
-    _stackWidget = new QStackedWidget(contentArea);
-    searchBar()->setParent(contentArea);
-    layout->addWidget(_stackWidget);
-    layout->addWidget(searchBar());
-    layout->setMargin(0);
-
-    // elide left is used because the most informative part of the session name is often
-    // the rightmost part
-    //
-    // this means you get entries looking like:
-    //
-    // ...dirA ...dirB ...dirC  ( helpful )
-    //
-    // instead of
-    //
-    // johnSmith@comput... johnSmith@comput...  ( not so helpful )
-    //
-
-    _listWidget->setTextElideMode( Qt::ElideLeft );
-    _listWidget->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
-    _listWidget->setDragDropMode(QAbstractItemView::DragDrop);
-    _splitter->addWidget(_listWidget);
-    _splitter->addWidget(contentArea);
-        
-    connect( _listWidget , SIGNAL(currentRowChanged(int)) , this , SLOT(rowChanged(int)) ); 
-}
-
-ListViewContainer::~ListViewContainer()
-{
-    _splitter->deleteLater();
-}
-
-QWidget* ListViewContainer::containerWidget() const
-{
-    return _splitter;
-}
-
-QWidget* ListViewContainer::activeView() const
-{
-    return _stackWidget->currentWidget();
-}
-
-QBrush ListViewContainer::randomItemBackground(int r)
-{
-    int i = r%6;
-
-    //and now for something truly unpleasant:
-    static const int r1[] = {255,190,190,255,190,255};
-    static const int r2[] = {255,180,180,255,180,255};
-    static const int b1[] = {190,255,190,255,255,190};
-    static const int b2[] = {180,255,180,255,255,180};
-    static const int g1[] = {190,190,255,190,255,255};
-    static const int g2[] = {180,180,255,180,255,255};
-
-    // hardcoded assumes item height is 32px
-    QLinearGradient gradient( QPoint(0,0) , QPoint(0,32) );
-    gradient.setColorAt(0,QColor(r1[i],g1[i],b1[i],100));
-    gradient.setColorAt(0.5,QColor(r2[i],g2[i],b2[i],100));
-    gradient.setColorAt(1,QColor(r1[i],g1[i],b1[i],100));
-    return QBrush(gradient);
-}
-
-void ListViewContainer::addViewWidget( QWidget* view , int )
-{
-    _stackWidget->addWidget(view);
-
-    ViewProperties* properties = viewProperties(view);
-
-    QListWidgetItem* item = new QListWidgetItem(_listWidget);
-    item->setText( properties->title() );
-    item->setIcon( properties->icon() );
-
-    const int randomIndex = _listWidget->count();
-    item->setData( Qt::BackgroundRole , randomItemBackground(randomIndex) );
-   
-    connect( properties , SIGNAL(titleChanged(ViewProperties*)) , this , SLOT(updateTitle(ViewProperties*)));
-    connect( properties , SIGNAL(iconChanged(ViewProperties*)) , this , SLOT(updateIcon(ViewProperties*)));
-}
-
-void ListViewContainer::removeViewWidget( QWidget* view )
-{
-    if (!_stackWidget)
-        return;
-    int index = _stackWidget->indexOf(view);
-    _stackWidget->removeWidget(view);
-    delete _listWidget->takeItem( index );
-}
-
-void ListViewContainer::setActiveView( QWidget* view )
-{
-    _stackWidget->setCurrentWidget(view);
-    _listWidget->setCurrentRow(_stackWidget->indexOf(view));
-}
-
-void ListViewContainer::rowChanged( int row )
-{
-    // row may be -1 if the last row has been removed from the model
-    if ( row >= 0 )
-    {
-        _stackWidget->setCurrentIndex( row );
-
-        emit activeViewChanged( _stackWidget->currentWidget() );
-    }
-}
-
-void ListViewContainer::updateTitle( ViewProperties* properties )
-{
-    QList<QWidget*> items = widgetsForItem(properties);
-    QListIterator<QWidget*> itemIter(items);
-
-    while ( itemIter.hasNext() )
-    {
-        int index = _stackWidget->indexOf( itemIter.next() );
-        _listWidget->item( index )->setText( properties->title() );
-    }
-}
-
-void ListViewContainer::updateIcon( ViewProperties* properties )
-{
-    QList<QWidget*> items = widgetsForItem(properties);
-    QListIterator<QWidget*> itemIter(items);
-
-    while ( itemIter.hasNext() )
-    {
-        int index = _stackWidget->indexOf( itemIter.next() );
-        _listWidget->item( index )->setIcon( properties->icon() );
-    }
-}
-
 #include "ViewContainer.moc"
