@@ -55,7 +55,7 @@
 #include <kfileitemactions.h>
 #include <kfileitemlistproperties.h>
 
-#include <knewmenu.h>
+#include <knewfilemenu.h>
 #include <konqmimedata.h>
 #include <konq_operations.h>
 #include <konq_popupmenu.h>
@@ -358,8 +358,9 @@ void FolderView::init()
 
     m_model->setFilterMode(ProxyModel::filterModeFromInt(m_filterType));
     m_model->setMimeTypeFilterList(m_filterFilesMimeList);
-    m_model->setFilterFixedString(m_filterFiles);
+    m_model->setFileNameFilter(m_filterFiles);
     m_model->setSortDirectoriesFirst(m_sortDirsFirst);
+    m_model->setDynamicSortFilter(true);
     m_model->sort(m_sortColumn != -1 ? m_sortColumn : KDirModel::Name, Qt::AscendingOrder);
 
     DirLister *lister = new DirLister(this);
@@ -417,6 +418,14 @@ void FolderView::init()
     connect(QApplication::clipboard(), SIGNAL(dataChanged()), SLOT(clipboardDataChanged()));
 }
 
+void FolderView::configChanged()
+{
+    //TODO: reload all other config values there
+    KConfigGroup cg = config();
+    setUrl(cg.readEntry("url", m_url));
+    m_dirModel->dirLister()->openUrl(m_url);
+}
+
 FolderView::~FolderView()
 {
     delete m_newMenu;
@@ -488,6 +497,7 @@ void FolderView::createConfigurationInterface(KConfigDialog *parent)
     if (isContainment()) {
         uiDisplay.lblCustomLabel->hide();
         uiDisplay.labelEdit->hide();
+        uiDisplay.headerTitle->hide();
     }
 
     uiDisplay.labelEdit->setText(m_titleText);
@@ -574,6 +584,27 @@ void FolderView::createConfigurationInterface(KConfigDialog *parent)
             }
         }
     }
+
+    connect(uiDisplay.labelEdit, SIGNAL(textChanged(QString)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.flowCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.sortCombo, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.sizeSlider, SIGNAL(sliderMoved(int)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.showPreviews, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.lockInPlace, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.alignToGrid, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.numLinesEdit, SIGNAL(valueChanged(int)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.colorButton, SIGNAL(changed(QColor)), parent, SLOT(settingsModified()));
+    connect(uiDisplay.drawShadows, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+
+    connect(uiFilter.filterType, SIGNAL(currentIndexChanged(int)), parent, SLOT(settingsModified()));
+    connect(uiFilter.filterFilesPattern, SIGNAL(textChanged(QString)), parent, SLOT(settingsModified()));
+    connect(uiFilter.filterFilesList->model(), SIGNAL(dataChanged(QModelIndex,QModelIndex)), parent, SLOT(settingsModified()));
+
+    connect(uiLocation.showDesktopFolder, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiLocation.showPlace, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiLocation.showCustomFolder, SIGNAL(toggled(bool)), parent, SLOT(settingsModified()));
+    connect(uiLocation.placesCombo, SIGNAL(currentIndexChagned(int)), parent, SLOT(settingsModified()));
+    connect(uiLocation.lineEdit, SIGNAL(textChanged(QString)), parent, SLOT(settingsModified()));
 }
 
 void FolderView::configAccepted()
@@ -641,7 +672,7 @@ void FolderView::configAccepted()
         cg.writeEntry("previewPlugins", m_previewPlugins);
         m_previewGenerator->setEnabledPlugins(m_previewPlugins);
 
-        //Chaning the preview plugins will also need a reload to work, so we need to preserve
+        //Changing the preview plugins will also need a reload to work, so we need to preserve
         //the icons position
         needReload = true;
         preserveIconPositions = true;
@@ -718,7 +749,7 @@ void FolderView::configAccepted()
     if (m_url != url || m_filterFiles != uiFilter.filterFilesPattern->text() ||
         m_filterFilesMimeList != selectedItems || m_filterType != filterType)
     {
-        m_model->setFilterFixedString(uiFilter.filterFilesPattern->text());
+        m_model->setFileNameFilter(uiFilter.filterFilesPattern->text());
         m_filterFiles = uiFilter.filterFilesPattern->text();
         m_filterFilesMimeList = selectedItems;
         m_filterType = filterType;
@@ -860,6 +891,9 @@ void FolderView::updateListViewState()
     m_listView->setDrawShadows(m_drawShadows);
     m_listView->setIconSize(iconSize());
     m_listView->setWordWrap(m_numTextLines > 1);
+    if ( m_iconView ) {
+        m_iconView->setTextLineCount(m_numTextLines);
+    }
 }
 
 void FolderView::updateIconViewState()
@@ -1374,7 +1408,7 @@ void FolderView::createActions()
         iconsMenuAction->setMenu(iconsMenu);
 
         // Create the new menu
-        m_newMenu = new KNewMenu(&m_actionCollection, QApplication::desktop(), "new_menu");
+        m_newMenu = new KNewFileMenu(&m_actionCollection, "new_menu", QApplication::desktop());
         connect(m_newMenu->menu(), SIGNAL(aboutToShow()), this, SLOT(aboutToShowCreateNew()));
 
         m_actionCollection.addAction("lock_icons", lockIcons);
@@ -1445,7 +1479,7 @@ QList<QAction*> FolderView::contextualActions()
 void FolderView::aboutToShowCreateNew()
 {
     if (m_newMenu) {
-        m_newMenu->slotCheckUpToDate();
+        m_newMenu->checkUpToDate();
         m_newMenu->setPopupFiles(m_url);
     }
 }
