@@ -1,5 +1,5 @@
 /*
- *   Copyright © 2008, 2009 Fredrik Höglund <fredrik@kde.org>
+ *   Copyright © 2008, 2009, 2010 Fredrik Höglund <fredrik@kde.org>
  *   Copyright © 2008 Rafael Fernández López <ereslibre@kde.org>
  *
  *   This library is free software; you can redistribute it and/or
@@ -39,7 +39,6 @@
 #include <QStyleOptionGraphicsItem>
 
 #include <KDirModel>
-#include <KDesktopFile>
 #include <KFileItemDelegate>
 #include <KGlobalSettings>
 #include <KIcon>
@@ -57,6 +56,7 @@
 #include "previewpluginsmodel.h"
 #include "tooltipwidget.h"
 #include "animator.h"
+#include "asyncfiletester.h"
 
 #include <Plasma/Containment>
 #include <Plasma/ContainmentActions>
@@ -83,6 +83,7 @@ IconView::IconView(QGraphicsWidget *parent)
       m_alignToGrid(false),
       m_wordWrap(false),
       m_popupShowPreview(true),
+      m_folderIsEmpty(false),
       m_flow(layoutDirection() == Qt::LeftToRight ? LeftToRight : RightToLeft),
       m_popupCausedWidget(0),
       m_dropOperation(0),
@@ -91,9 +92,6 @@ IconView::IconView(QGraphicsWidget *parent)
 {
     m_actionOverlay = new ActionOverlay(this);
 
-    // Added for later use
-    Q_UNUSED(I18N_NOOP("This folder is empty."))
-
     setAcceptHoverEvents(true);
     setAcceptDrops(true);
     setCacheMode(NoCache);
@@ -483,8 +481,9 @@ void IconView::listingStarted(const KUrl &url)
     Q_UNUSED(url)
 
     // Reset any error message that may have resulted from an earlier listing
-    if (!m_errorMessage.isEmpty()) {
+    if (!m_errorMessage.isEmpty() || m_folderIsEmpty) {
         m_errorMessage.clear();
+        m_folderIsEmpty = false;
         update();
     }
 
@@ -505,6 +504,14 @@ void IconView::listingCompleted()
     if (m_validRows == m_model->rowCount()) {
         emit busy(false);
     }
+
+    if (!m_model->rowCount() && !m_folderIsEmpty) {
+        m_folderIsEmpty = true;
+        update();
+    } else if (m_model->rowCount() && m_folderIsEmpty) {
+        m_folderIsEmpty = false;
+        update();
+    }
 }
 
 void IconView::listingCanceled()
@@ -534,6 +541,10 @@ void IconView::itemsDeleted(const KFileItemList &items)
         const QString path = m_dirModel->dirLister()->url().toLocalFile();
         listingError(KIO::buildErrorString(KIO::ERR_DOES_NOT_EXIST, path));
     }
+
+    if (!m_model->rowCount()) {
+        m_folderIsEmpty = true;
+    }
 }
 
 int IconView::columnsForWidth(qreal width) const
@@ -1227,6 +1238,11 @@ void IconView::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 
     if (!m_errorMessage.isEmpty()) {
         paintMessage(painter, cr, m_errorMessage, KIcon("dialog-error"));
+    } else if (m_folderIsEmpty) {
+        Plasma::Containment *containment = qobject_cast<Plasma::Containment*>(parentWidget());
+        if (!containment || !containment->isContainment()) {
+            paintMessage(painter, cr, i18n( "This folder is empty." ), KIcon() );
+        }
     }
 }
 
@@ -1530,9 +1546,33 @@ void IconView::focusOutEvent(QFocusEvent *event)
     markAreaDirty(visibleArea());
 }
 
-void IconView::triggerToolTip(ToolTipType type)
+// Updates the tooltip for the hovered index
+void IconView::updateToolTip()
 {
-    if (type == FolderTip && m_hoveredIndex.isValid()) {
+    // Close the popup view if one is open
+    m_toolTipShowTimer.stop();
+    m_popupCausedWidget = 0;
+    if (m_popupView) {
+        m_popupView->delayedHide();
+    }
+
+    // Don't show file tips when the user is dragging something over the view
+    if (!m_hoverDrag) {
+        m_toolTipWidget->updateToolTip(m_hoveredIndex, mapFromViewport(visualRect(m_hoveredIndex)));
+    } else {
+        m_toolTipWidget->updateToolTip(QModelIndex(), QRect());
+    }
+}
+
+void IconView::checkIfFolderResult(const QModelIndex &index, bool isFolder)
+{
+    m_toolTipShowTimer.stop();
+    if (index != m_hoveredIndex) {
+        return;
+    }
+
+    // Start the timer if the index turned out to be a folder
+    if (isFolder && index.isValid()) {
         // Use a longer delay if we don't have any popup view open or if it's been
         // longer than 1.5 seconds since the last popup view was opened or closed.
         if ((m_popupView && m_hoveredIndex != m_popupIndex) ||
@@ -1541,80 +1581,8 @@ void IconView::triggerToolTip(ToolTipType type)
         } else {
             m_toolTipShowTimer.start(1000, this);
         }
-    } else {
-        // Close the popup view if one is open
-        m_toolTipShowTimer.stop();
-        m_popupCausedWidget = 0;
-        m_popupUrl = KUrl();
-        if (m_popupView) {
-            m_popupView->delayedHide();
-        }
-        // Don't show file tips when the user is dragging something over the view
-        if (!m_hoverDrag) {
-            m_toolTipWidget->updateToolTip(m_hoveredIndex, mapFromViewport(visualRect(m_hoveredIndex)));
-        } else {
-            m_toolTipWidget->updateToolTip(QModelIndex(), QRect());
-        }
-    }
-}
-
-// Updates the tooltip for the hovered index
-// causedWidget is the widget that received the mouse event that triggered the update
-void IconView::updateToolTip(QWidget *causedWidget)
-{
-    if (!m_hoveredIndex.isValid()) {
-        triggerToolTip(FileTip); // Will close any open tips
-        return;
-    }
-
-    if (m_popupView && m_hoveredIndex == m_popupIndex) {
-        // If we're already showing a popup view for this index
-        return;
-    }
-
-    // Decide if we're going to show a popup view or a regular tooltip
-    IconView::ToolTipType type = IconView::FileTip;
-    bool delayedResult = false;
-
-    KFileItem item = m_model->itemForIndex(m_hoveredIndex);
-    KUrl url = item.targetUrl();
-
-    if (item.isDir()) {
-        type = IconView::FolderTip;
-    } else if (item.isDesktopFile()) {
-        // Check if the desktop file is a link to a local folder
-        KDesktopFile file(url.path());
-        if (file.readType() == "Link") {
-            url = file.readUrl();
-            if (url.isLocalFile()) {
-                KFileItem destItem(KFileItem::Unknown, KFileItem::Unknown, url);
-                type = destItem.isDir() ? IconView::FolderTip : IconView::FileTip;
-            } else if (KProtocolInfo::protocolClass(url.protocol()) == QString(":local")) {
-                KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
-                job->setSide(KIO::StatJob::SourceSide); // We will only read the file
-                connect(job, SIGNAL(result(KJob*)), SLOT(statResult(KJob*)));
-                delayedResult = true;
-            }
-        }
-    } 
-
-    m_popupUrl = url;
-    m_popupCausedWidget = causedWidget;
-
-    if (!delayedResult) {
-        triggerToolTip(type);
-    }
-}
-
-void IconView::statResult(KJob *job)
-{
-    if (!job->error()) {
-        KIO::StatJob *statJob = static_cast<KIO::StatJob*>(job);
-        if (statJob->statResult().isDir()) {
-            triggerToolTip(IconView::FolderTip);
-        } else {
-            triggerToolTip(IconView::FileTip);
-        }
+    } else if (m_popupView) {
+        m_popupView->delayedHide();
     }
 }
 
@@ -1626,7 +1594,7 @@ void IconView::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
         m_hoveredIndex = index;
         markAreaDirty(visualRect(index));
     }
-    updateToolTip(event->widget());
+    updateToolTip();
 }
 
 void IconView::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
@@ -1637,7 +1605,7 @@ void IconView::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
         emit left(m_hoveredIndex);
         markAreaDirty(visualRect(m_hoveredIndex));
         m_hoveredIndex = QModelIndex();
-        updateToolTip(event->widget());
+        updateToolTip();
     }
 
     m_actionOverlay->forceHide(ActionOverlay::FadeOut);
@@ -1656,20 +1624,48 @@ void IconView::hoverMoveEvent(QGraphicsSceneHoverEvent *event)
         markAreaDirty(visualRect(index));
         markAreaDirty(visualRect(m_hoveredIndex));
         m_hoveredIndex = index;
-        updateToolTip(event->widget());
+        updateToolTip();
     }
 }
 
 void IconView::keyPressEvent(QKeyEvent *event)
 {
-    if (m_columns == 0)        //The layout isn't done until items are actually inserted into the model, so until that happens, m_columns will be 0
+    if (m_columns == 0)  {
+        //The layout isn't done until items are actually inserted into the model, so until that happens, m_columns will be 0
         return;
+    }
 
     int hdirection = 0;
     int vdirection = 0;
 
     QModelIndex currentIndex = m_selectionModel->currentIndex();
 
+    if (!event->text().isEmpty()) {
+        bool sameKeyWasPressed = m_searchQuery.endsWith(event->text());
+        m_searchQuery.append(event->text());
+        m_searchQueryTimer.start(1500, this);   //clears search query when the user doesn't press any key
+
+        // First try to match the exact icon string
+        QModelIndexList matches = m_model->match(currentIndex, Qt::DisplayRole, m_searchQuery, 1,
+                                                 Qt::MatchFixedString | Qt::MatchWrap);
+
+        if (matches.count() <= 0) {
+            // Exact match failed, try matching the beginning of the icon string
+            matches = m_model->match(currentIndex, Qt::DisplayRole, m_searchQuery, 1,
+                                     Qt::MatchStartsWith | Qt::MatchWrap);
+
+            if (matches.count() <= 0 && sameKeyWasPressed) {
+                // Didn't even match beginning, try next icon string starting with the same letter
+                matches = m_model->match(currentIndex.sibling(currentIndex.row()+1, currentIndex.column()),
+                        Qt::DisplayRole, event->text(), 1, Qt::MatchStartsWith | Qt::MatchWrap);
+            }
+        }
+
+        if (matches.count() > 0) {
+            selectIcon(matches.at(0));
+        }
+    }
+
     switch (event->key()) {
     case Qt::Key_Home:
         selectFirstIcon();
@@ -1847,8 +1843,6 @@ void IconView::mousePressEvent(QGraphicsSceneMouseEvent *event)
         m_pressedIndex = QModelIndex();
         m_buttonDownPos = pos;
 
-        const Plasma::Containment *parent = qobject_cast<Plasma::Containment*>(parentWidget());
-
         // If a containment action is assigned to the left mouse button,
         // give it precedence over rubberband-selections
         Plasma::Containment *containment = qobject_cast<Plasma::Containment*>(parentWidget());
@@ -1994,7 +1988,10 @@ void IconView::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 
 void IconView::wheelEvent(QGraphicsSceneWheelEvent *event)
 {
-    if ((event->modifiers() & Qt::CTRL) || (event->orientation() == Qt::Horizontal)) {
+    Plasma::Containment *containment = qobject_cast<Plasma::Containment*>(parentWidget());
+    if ((containment && containment->isContainment() && !m_scrollBar->isVisible()) ||
+        (event->modifiers() & Qt::CTRL) ||
+        (event->orientation() == Qt::Horizontal)) {
         // Let the event propagate to the parent widget
         event->ignore();
         return;
@@ -2075,7 +2072,13 @@ void IconView::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
             dirtyRect |= visualRect(index);
         }
     }
-    updateToolTip(event->widget());
+
+    // Open a popup view for this index if it's a folder
+    if (!m_popupView || m_hoveredIndex != m_popupIndex) {
+        // If we're not already showing a popup view for this index
+        m_popupCausedWidget = event->widget();
+        AsyncFileTester::checkIfFolder(m_hoveredIndex, this, "checkIfFolderResult");
+    }
 
     markAreaDirty(dirtyRect);
     event->setAccepted(!event->mimeData()->hasFormat(appletMimeType));
@@ -2110,7 +2113,8 @@ void IconView::createDropActions(const KUrl::List &urls, QActionGroup *actions)
         // Unfortunately QImageReader::supportedImageFormats() returns some formats in upper case
         // and others in lower case, but all the important ones are in lower case.
         const QByteArray suffix = QFileInfo(urls.first().fileName()).suffix().toLower().toUtf8();
-        if (mimeName.startsWith("image/") || QImageReader::supportedImageFormats().contains(suffix)) {
+        if (mimeName.startsWith(QLatin1String("image/")) ||
+                QImageReader::supportedImageFormats().contains(suffix)) {
             QAction *action = new QAction(i18n("Set as &Wallpaper"), actions);
             action->setData("internal:folderview:set-as-wallpaper");
             action->setIcon(KIcon("preferences-desktop-wallpaper"));
@@ -2466,6 +2470,9 @@ QStyleOptionViewItemV4 IconView::viewOptions() const
 
 void IconView::selectIcon(QModelIndex index)
 {
+    if (!index.isValid()) {
+        return;
+    }
     repaintSelectedIcons();
     m_selectionModel->select(index, QItemSelectionModel::ClearAndSelect);
     m_selectionModel->setCurrentIndex(index, QItemSelectionModel::NoUpdate);
@@ -2476,8 +2483,8 @@ void IconView::selectIcon(QModelIndex index)
 
 void IconView::selectFirstOrLastIcon(bool firstIcon)
 {
-    int minVertical;
-    int minHorizontal;
+    int minVertical=0;
+    int minHorizontal=0;
     int dirn=1;    //Useful in calculations as it stores whether view is LeftToRight or RightToLeft
     int isFirst = firstIcon ? 1 : -1;    //Useful in calculations to decide whether to select First or Last icon
     QModelIndex toSelect;
@@ -2561,6 +2568,54 @@ void IconView::popupCloseRequested()
     }
 }
 
+void IconView::openPopup(const QModelIndex &index)
+{
+    if (m_popupView && m_popupIndex == index) {
+        // The popup is already showing the hovered index
+        return;
+    }
+
+    if (m_popupView && m_popupView->dragInProgress()) {
+        // Don't delete the popup view when a drag is in progress
+        return;
+    }
+
+    Plasma::ToolTipManager::self()->hide(m_toolTipWidget);
+    delete m_popupView;
+
+    if (QApplication::activePopupWidget() || QApplication::activeModalWidget()) {
+        // Don't open a popup view when a menu or similar widget is being shown 
+        return;
+    }
+
+    // Make sure the index is valid
+    if (!index.isValid()) {
+        return;
+    }
+
+    const QPointF viewPos = mapFromViewport(visualRect(index)).center();
+    const QPoint scenePos = mapToScene(viewPos).toPoint();
+    QGraphicsView *gv = 0;
+
+    if (m_popupCausedWidget) {
+        gv = qobject_cast<QGraphicsView*>(m_popupCausedWidget->parentWidget());
+    } else {
+        // We position the popup relative to the view under the mouse cursor
+        foreach (QGraphicsView *view, scene()->views()) {
+            if (view->underMouse()) {
+                gv = view;
+                break;
+            }
+        }
+    }
+
+    const QPoint pos = gv ? gv->mapToGlobal(gv->mapFromScene(scenePos)) : QPoint();
+    m_popupIndex = index;
+    m_popupView = new PopupView(m_popupIndex, pos, m_popupShowPreview, m_popupPreviewPlugins, this);
+    connect(m_popupView, SIGNAL(destroyed(QObject*)), SIGNAL(popupViewClosed()));
+    connect(m_popupView, SIGNAL(requestClose()), SLOT(popupCloseRequested()));
+}
+
 void IconView::timerEvent(QTimerEvent *event)
 {
     AbstractItemView::timerEvent(event);
@@ -2619,48 +2674,10 @@ void IconView::timerEvent(QTimerEvent *event)
         }
     } else if (event->timerId() == m_toolTipShowTimer.timerId()) {
         m_toolTipShowTimer.stop();
-
-        if (m_popupView && m_popupIndex == m_hoveredIndex) {
-            // The popup is already showing the hovered index
-            return;
-        }
-
-        if (m_popupView && m_popupView->dragInProgress()) {
-            // Don't delete the popup view when a drag is in progress
-            return;
-        }
-
-        Plasma::ToolTipManager::self()->hide(m_toolTipWidget);
-        delete m_popupView;
-
-        if (QApplication::activePopupWidget() || QApplication::activeModalWidget()) {
-            // Don't open a popup view when a menu or similar widget is being shown 
-            return;
-        }
-
-        if (!m_popupUrl.isEmpty()) {
-            const QPointF viewPos = mapFromViewport(visualRect(m_hoveredIndex)).center();
-            const QPoint scenePos = mapToScene(viewPos).toPoint();
-            QGraphicsView *gv = 0;
-
-            if (m_popupCausedWidget) {
-                gv = qobject_cast<QGraphicsView*>(m_popupCausedWidget->parentWidget());
-            } else {
-                // We position the popup relative to the view under the mouse cursor
-                foreach (QGraphicsView *view, scene()->views()) {
-                    if (view->underMouse()) {
-                        gv = view;
-                        break;
-                    }
-                }
-            }
-
-            const QPoint pos = gv ? gv->mapToGlobal(gv->mapFromScene(scenePos)) : QPoint();
-            m_popupView = new PopupView(m_popupUrl, pos, m_popupShowPreview, m_popupPreviewPlugins, this);
-            connect(m_popupView, SIGNAL(destroyed(QObject*)), SIGNAL(popupViewClosed()));
-            connect(m_popupView, SIGNAL(requestClose()), SLOT(popupCloseRequested()));
-            m_popupIndex = m_hoveredIndex;
-        }
+        openPopup(m_hoveredIndex);
+    } else if (event->timerId() == m_searchQueryTimer.timerId()) {
+        m_searchQuery.clear();
+        m_searchQueryTimer.stop();
     }
 }
 
