@@ -56,13 +56,15 @@
 #include <Plasma/Applet>
 #include <Plasma/BusyWidget>
 #include <Plasma/FrameSvg>
-
-#ifdef Q_WS_X11
-#  include <QX11Info>
-#endif
+#include <Plasma/Theme>
+#include <Plasma/WindowEffects>
 
 
-PopupView::PopupView(const KUrl &url, const QPoint &pos, const IconView *parentView)
+QTime PopupView::s_lastOpenClose;
+
+PopupView::PopupView(const QModelIndex &index, const QPoint &pos,
+                     const bool &showPreview, const QStringList &previewPlugins,
+                     const IconView *parentView)
     : QWidget(0, Qt::X11BypassWindowManagerHint),
       m_view(0),
       m_parentView(parentView),
@@ -70,23 +72,43 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos, const IconView *parentV
       m_iconView(0),
       m_dirModel(0),
       m_model(0),
-      m_url(url),
       m_actionCollection(this),
       m_newMenu(0),
       m_itemActions(0),
-      m_showingMenu(false)
+      m_showingMenu(false),
+      m_showPreview(showPreview),
+      m_previewPlugins(previewPlugins)
 {
     setAttribute(Qt::WA_TranslucentBackground);
+#ifdef Q_WS_X11
+    if (KWindowSystem::compositingActive()) {
+        setAttribute(Qt::WA_NoSystemBackground, false);
+    }
+#endif
+
+#ifdef Q_WS_WIN
+    setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint | Qt::Tool);
+#endif
+
     KWindowSystem::setState(effectiveWinId(), NET::SkipTaskbar | NET::SkipPager);
 
     setAcceptDrops(true);
 
     QPalette pal = palette();
     pal.setColor(backgroundRole(), Qt::transparent);
+    pal.setColor(QPalette::Text, Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));
     setPalette(pal);
 
+    KFileItem item = static_cast<const ProxyModel*>(index.model())->itemForIndex(index);
+    if (item.isDesktopFile()) {
+        KDesktopFile file(item.localPath());
+        m_url = file.readUrl();
+    } else {
+        m_url = item.targetUrl();
+    }
+
     m_background = new Plasma::FrameSvg(this);
-    m_background->setImagePath("dialogs/background");
+    m_background->setImagePath("widgets/tooltip");
 
     int left   = m_background->marginSize(Plasma::LeftMargin);
     int top    = m_background->marginSize(Plasma::TopMargin);
@@ -97,7 +119,7 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos, const IconView *parentV
 
     resize(parentView->sizeForRowsColumns(2, 3) + QSize(left + right, top + bottom));
 
-    const QRect available = QApplication::desktop()->availableGeometry();
+    const QRect available = QApplication::desktop()->availableGeometry(pos);
     QPoint pt = pos;
 
     if (pt.x() + width() > available.right()) {
@@ -114,15 +136,19 @@ PopupView::PopupView(const KUrl &url, const QPoint &pos, const IconView *parentV
         pt.ry() = available.top();
     }
 
+    Plasma::WindowEffects::overrideShadow(winId(), true);
+
     move(pt);
     show();
 
     QTimer::singleShot(10, this, SLOT(init()));
+    s_lastOpenClose.restart();
 }
 
 PopupView::~PopupView()
 {
     delete m_newMenu;
+    s_lastOpenClose.restart();
 }
 
 void PopupView::delayedHide()
@@ -164,6 +190,7 @@ void PopupView::init()
     m_model = new ProxyModel(this);
     m_model->setSourceModel(m_dirModel);
     m_model->setSortLocaleAware(true);
+    m_model->setDynamicSortFilter(true);
     m_model->setFilterCaseSensitivity(Qt::CaseInsensitive);
     m_model->setParseDesktopFiles(m_url.protocol() == "desktop");
     m_model->sort(KDirModel::Name, Qt::AscendingOrder);
@@ -176,7 +203,7 @@ void PopupView::init()
     m_iconView->setItemDelegate(m_delegate);
     m_iconView->setSelectionModel(m_selectionModel);
     m_iconView->setFont(m_parentView->font());
-    m_iconView->setPalette(m_parentView->palette());
+    m_iconView->setPalette(palette());
     m_iconView->setDrawShadows(m_parentView->drawShadows());
     m_iconView->setIconSize(m_parentView->iconSize());
     m_iconView->setGridSize(m_parentView->gridSize());
@@ -188,11 +215,10 @@ void PopupView::init()
     connect(m_iconView, SIGNAL(busy(bool)), SLOT(setBusy(bool)));
     connect(m_iconView, SIGNAL(popupViewClosed()), SLOT(maybeClose()));
 
-    // TODO: The preview settings should be inherited from the parent view
     FolderViewAdapter *adapter = new FolderViewAdapter(m_iconView);
     m_previewGenerator = new KFilePreviewGenerator(adapter, m_model);
-    m_previewGenerator->setPreviewShown(true);
-    m_previewGenerator->setEnabledPlugins(QStringList() << "imagethumbnail" << "jpegthumbnail");
+    m_previewGenerator->setPreviewShown(m_showPreview);
+    m_previewGenerator->setEnabledPlugins(m_previewPlugins);
 
     m_iconView->setGeometry(contentsRect());
     m_iconView->show();
@@ -238,13 +264,18 @@ void PopupView::createActions()
     connect(trash, SIGNAL(triggered(Qt::MouseButtons, Qt::KeyboardModifiers)),
             SLOT(moveToTrash(Qt::MouseButtons, Qt::KeyboardModifiers)));
 
+    KAction *emptyTrash = new KAction(KIcon("trash-empty"), i18n("&Empty Trash Bin"), this);
+    KConfig trashConfig("trashrc", KConfig::SimpleConfig);
+    emptyTrash->setEnabled(!trashConfig.group("Status").readEntry("Empty", true));
+    connect(emptyTrash, SIGNAL(triggered()), SLOT(emptyTrashBin()));
+
     KAction *del = new KAction(i18n("&Delete"), this);
     del->setIcon(KIcon("edit-delete"));
     del->setShortcut(Qt::SHIFT + Qt::Key_Delete);
     connect(del, SIGNAL(triggered()), SLOT(deleteSelectedIcons()));
 
     // Create the new menu
-    m_newMenu = new KNewMenu(&m_actionCollection, this, "new_menu");
+    m_newMenu = new KNewFileMenu(&m_actionCollection, "new_menu", this);
     connect(m_newMenu->menu(), SIGNAL(aboutToShow()), this, SLOT(aboutToShowCreateNew()));
 
     m_actionCollection.addAction("undo", undo);
@@ -255,10 +286,12 @@ void PopupView::createActions()
     m_actionCollection.addAction("rename", rename);
     m_actionCollection.addAction("trash", trash);
     m_actionCollection.addAction("del", del);
+    m_actionCollection.addAction("empty_trash", emptyTrash);
 }
 
 void PopupView::contextMenuRequest(QWidget *widget, const QPoint &screenPos)
 {
+    Q_UNUSED(widget)
     // contextMenuRequest is only called from the icon view, which is created in init()
     // which mean m_model should always be initialized
     Q_ASSERT(m_model);
@@ -366,14 +399,14 @@ void PopupView::copy()
 
 void PopupView::paste()
 {
-    KonqOperations::doPaste(this, m_url);
+    KonqOperations::doPaste(QApplication::desktop(), m_url);
 }
 
 void PopupView::pasteTo()
 {
     KUrl::List urls = selectedUrls();
     Q_ASSERT(urls.count() == 1);
-    KonqOperations::doPaste(this, urls.first());
+    KonqOperations::doPaste(QApplication::desktop(), urls.first());
 }
 
 void PopupView::moveToTrash(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers)
@@ -383,14 +416,14 @@ void PopupView::moveToTrash(Qt::MouseButtons buttons, Qt::KeyboardModifiers modi
         KonqOperations::Operation op = (modifiers & Qt::ShiftModifier) ?
                 KonqOperations::DEL : KonqOperations::TRASH;
 
-        KonqOperations::del(this, op, selectedUrls());
+        KonqOperations::del(QApplication::desktop(), op, selectedUrls());
     }
 }
 
 void PopupView::deleteSelectedIcons()
 {
     if (!m_iconView->renameInProgress()) {
-        KonqOperations::del(this, KonqOperations::DEL, selectedUrls());
+        KonqOperations::del(QApplication::desktop(), KonqOperations::DEL, selectedUrls());
     }
 }
 
@@ -405,23 +438,35 @@ void PopupView::activated(const QModelIndex &index)
     const KFileItem item = m_model->itemForIndex(index);
     item.run();
 
-    hide();
-    deleteLater();
+    closeThisAndParentPopup();
 }
 
 void PopupView::setBusy(bool busy)
 {
+    m_busy = busy;
     if (busy && !m_busyWidget) {
-        const int size = qMin(width(), height()) * .3;
-        m_busyWidget = new Plasma::BusyWidget;
-        m_busyWidget->setGeometry(QStyle::alignedRect(layoutDirection(), Qt::AlignCenter, QSize(size, size), contentsRect()));
-        m_scene->addItem(m_busyWidget);
+        QTimer::singleShot(100, this, SLOT(createBusyWidgetIfNeeded()));
     } else {
         delete m_busyWidget;
         m_busyWidget = 0;
     }
 }
 
+void PopupView::createBusyWidgetIfNeeded()
+{
+    if (m_busy && !m_busyWidget) {
+        const int size = qMin(width(), height()) * .3;
+        m_busyWidget = new Plasma::BusyWidget;
+        m_busyWidget->setGeometry(QStyle::alignedRect(layoutDirection(), Qt::AlignCenter, QSize(size, size), contentsRect()));
+        m_scene->addItem(m_busyWidget);
+    }
+}
+
+void PopupView::emptyTrashBin()
+{
+    KonqOperations::emptyTrash(QApplication::desktop());
+}
+
 void PopupView::undoTextChanged(const QString &text)
 {
     if (QAction *action = m_actionCollection.action("undo")) {
@@ -432,7 +477,7 @@ void PopupView::undoTextChanged(const QString &text)
 void PopupView::aboutToShowCreateNew()
 {
     if (m_newMenu) {
-        m_newMenu->slotCheckUpToDate();
+        m_newMenu->checkUpToDate();
         m_newMenu->setPopupFiles(m_url);
     }
 }
@@ -446,11 +491,12 @@ void PopupView::resizeEvent(QResizeEvent *event)
     if (m_view) {
         m_view->setGeometry(contentsRect());
     }
-#ifdef Q_WS_X11
-    if (!QX11Info::isCompositingManagerRunning()) {
+
+    if (KWindowSystem::compositingActive()) {
+        Plasma::WindowEffects::enableBlurBehind(winId(), true, m_background->mask());
+    } else {
         setMask(m_background->mask());
     }
-#endif
 }
 
 void PopupView::paintEvent(QPaintEvent *event)
@@ -474,6 +520,12 @@ void PopupView::contextMenuEvent(QContextMenuEvent *event)
         createActions();
     }
 
+    KFileItem rootItem = m_model->itemForIndex(QModelIndex());
+    //The root item is invalid (non-existent)
+    if (rootItem.isNull()) {
+        return;
+    }
+    
     QMenu menu;
     menu.addAction(m_actionCollection.action("new_menu"));
     menu.addSeparator();
@@ -485,7 +537,6 @@ void PopupView::contextMenuEvent(QContextMenuEvent *event)
     if (!m_itemActions) {
         // Create a new KFileItem to prevent the target URL in the root item
         // from being used. In this case we want the configured URL instead.
-        KFileItem rootItem = m_model->itemForIndex(QModelIndex());
         KFileItem item(rootItem.mode(), rootItem.permissions(), m_url);
 
         KFileItemListProperties itemList(KFileItemList() << item);
@@ -495,6 +546,10 @@ void PopupView::contextMenuEvent(QContextMenuEvent *event)
     }
     menu.addAction(m_itemActions->preferredOpenWithAction(QString()));
 
+    if (m_url.protocol() == "trash") {
+        menu.addAction(m_actionCollection.action("empty_trash"));
+    }
+
     m_showingMenu = true;
     menu.exec(event->globalPos());
     m_showingMenu = false;
@@ -525,6 +580,12 @@ void PopupView::maybeClose()
     }
 }
 
+void PopupView::closeThisAndParentPopup() {
+    hide();
+    deleteLater();
+    callOnParent("closeThisAndParentPopup");
+}
+
 void PopupView::cancelHideTimer()
 {
     m_hideTimer.stop();
@@ -573,7 +634,7 @@ void PopupView::dragEnterEvent(QDragEnterEvent *event)
 
 void PopupView::dragLeaveEvent(QDragLeaveEvent *event)
 {
-    if (!m_iconView->popupVisible()) {
+    if (!m_iconView || !m_iconView->popupVisible()) {
         maybeClose();
     }
 
